package com.ruoyi.common.thread.sai;

import com.github.stuxuhai.jpinyin.PinyinException;
import com.github.stuxuhai.jpinyin.PinyinFormat;
import com.github.stuxuhai.jpinyin.PinyinHelper;
import com.ruoyi.common.sendEmail.IMailService;
import com.ruoyi.project.production.domain.TSaiApply;
import com.ruoyi.project.system.domain.SysMessage;
import com.ruoyi.project.system.service.ISysMessageService;
import org.activiti.engine.task.Task;

import java.text.SimpleDateFormat;
import java.util.Date;

/**
 * SAI开项申请流程邮件通知线程
 *
 * @author Wang Zi Wen
 * @email wangggziwen@163.com
 * @date 2023/04/10 10:35:50
 */
public class SaiApplyMailThread implements Runnable {

    private IMailService mailService;
    private ISysMessageService sysMessageService;
    private SysMessage message;
    private String email;
    private Task task;
    private TSaiApply tSaiApply;
    private String staffName;
    private String staffNameEN;
    private String loginName;

    public SaiApplyMailThread() {}

    public SaiApplyMailThread(IMailService mailService, ISysMessageService sysMessageService, SysMessage message, String email, Task task, TSaiApply tSaiApply, String staffName, String loginName) throws PinyinException {
        this.mailService = mailService;
        this.sysMessageService = sysMessageService;
        this.email = email;
        this.message = message;
        this.task = task;
        this.tSaiApply = tSaiApply;
        this.staffName = staffName;
        this.staffNameEN = PinyinHelper.convertToPinyinString(staffName.trim(), " ", PinyinFormat.WITHOUT_TONE);
        this.loginName =  loginName;
    }

    @Override
    public void run() {
        this.sendMail();
        sysMessageService.insertSysMessage(message);
    }

    private void sendMail() {
        String jumpUrl = "/production/sai/apply";
        //写html开始内容
        String start = "<!DOCTYPE html><html><head><meta charset=\"utf-8\"><title></title></head><body><div style=\"background-color:#ECECEC; padding: 35px;\">" +
                "<table cellpadding=\"0\" align=\"center\"" +
                "style=\"width: 600px; margin: 0px auto; text-align: left; position: relative; border-top-left-radius: 5px; border-top-right-radius: 5px; border-bottom-right-radius: 5px; border-bottom-left-radius: 5px; font-size: 14px; font-family:微软雅黑, 黑体; line-height: 1.5; box-shadow: rgb(153, 153, 153) 0px 0px 5px; border-collapse: collapse; background-position: initial initial; background-repeat: initial initial;background:#fff;\">" +
                "<tbody><tr><th valign=\"middle\" style=\"height: 25px; line-height: 25px; padding: 15px 35px; border-bottom-width: 1px; border-bottom-style: solid; border-bottom-color: #42a3d3; background-color: #49bcff; border-top-left-radius: 5px; border-top-right-radius: 5px; border-bottom-right-radius: 0px; border-bottom-left-radius: 0px;\">" +
                "<font face=\"微软雅黑\" size=\"5\" style=\"color: rgb(255, 255, 255); \">CPMS管理系统 </font><font face=\"微软雅黑\" size=\"3\" style=\"color: rgb(255, 255, 255); \">CPMS System</font></th></tr>";
        //表html中间内容
        String prime = "";
        String center = "<tr><td><div style=\"padding:25px 35px 40px; background-color:#fff;\"><h2 style=\"margin: 5px 0px; \">" +
                "<font color=\"#333333\" style=\"line-height: 20px; \"><font style=\"line-height: 22px; \" size=\"4\">" +
                "亲爱的staffName</font><br><font style=\"line-height: 22px; \" size=\"4\">" +
                "Dear staffNameEN(loginName)</font></font></h2>" +
                "<p>您有新的SAI开项申请待办任务，详情如下：<br>" +
                "You have a new to-do task:<br>" +
                "流程编号：<b>processId</b><br>" +
                "任务名称：<b>taskName</b><br>" +
                "问题描述：<b>taskDescription</b><br>" +
                "申请时间：<b>taskApplyDate</b><br>" +
                "请登录<a href=\"https://cpms.basf-ypc.net.cn/cpms/index.html#jumpUrl\">CPMS管理系统</a>查看。<br>" +
                "Please log in the <a href=\"https://cpms.basf-ypc.net.cn/cpms/index.html#jumpUrl\">CPMS</a> to handle it.</p>" +
                "<p align=\"right\">date</p>" +
                "<div style=\"width:700px;margin:0 auto;\">" +
                "<div style=\"padding:10px 10px 0;border-top:1px solid #ccc;color:#747474;margin-bottom:20px;line-height:1.3em;font-size:12px;\">" +
                "<p>此为系统邮件，请勿回复<br>This e-Mail is an automatic reminder sent by CPMS, please do not reply</p>" +
                "</div></div></div></td></tr>";
        String one = center.replaceFirst("staffName", staffName);
        String two = one.replaceFirst("staffNameEN", staffNameEN);
        String three = two.replaceFirst("loginName", loginName);
        String four = three.replaceFirst("processId", tSaiApply.getProcessId());
        String five = four.replaceFirst("taskName", task.getName());
        String six = five.replaceFirst("taskDescription", tSaiApply.getDescription());
        String seven = six.replaceFirst("taskApplyDate", new SimpleDateFormat("yyyy-MM-dd hh:mm:ss").format(tSaiApply.getApplyDate()));
        String result = seven.replaceFirst("date", String.valueOf(new Date())).replaceFirst("jumpUrl",jumpUrl).replaceFirst("jumpUrl",jumpUrl);
        prime = prime + result;
        //写html结尾内容
        String end = "</tbody></table></div></body></html>";
        //拼接html
        String html = start + prime + end;
        mailService.sendHtmlMail(email, "CPMS：您有新的SAI开项申请待办任务", html);
    }
}
