package com.ruoyi.project.affair.controller;

import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import com.alibaba.fastjson.JSON;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.file.ExcelUtils;
import com.ruoyi.project.system.domain.SysDept;
import com.ruoyi.project.system.domain.SysDictData;
import com.ruoyi.project.system.service.ISysDeptService;
import com.ruoyi.project.system.service.ISysDictTypeService;
import org.apache.poi.ss.usermodel.*;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import com.ruoyi.framework.aspectj.lang.annotation.Log;
import com.ruoyi.framework.aspectj.lang.enums.BusinessType;
import com.ruoyi.project.affair.domain.TIntercom;
import com.ruoyi.project.affair.service.ITIntercomService;
import com.ruoyi.framework.web.controller.BaseController;
import com.ruoyi.framework.web.domain.AjaxResult;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.framework.web.page.TableDataInfo;
import org.springframework.web.multipart.MultipartFile;

/**
 * 对讲机管理Controller
 *
 * @author ruoyi
 * @date 2020-12-02
 */
@RestController
@RequestMapping("/affair/intercom")
public class TIntercomController extends BaseController
{
    @Autowired
    private ITIntercomService tIntercomService;

    @Autowired
    private ISysDeptService iSysDeptService;

    @Autowired
    private ISysDictTypeService iSysDictTypeService;

    /**
     * 查询对讲机管理列表
     */
    @PreAuthorize("@ss.hasPermi('affair:intercom:list')")
    @GetMapping("/list")
    public TableDataInfo list(TIntercom tIntercom)
    {
        startPage();
        List<TIntercom> list = tIntercomService.selectTIntercomList(tIntercom);
        return getDataTable(list);
    }

    /**
     * 导出对讲机管理列表
     */
    @PreAuthorize("@ss.hasPermi('affair:intercom:export')")
    @Log(title = "对讲机管理", businessType = BusinessType.EXPORT)
    @GetMapping("/export")
    public AjaxResult export(TIntercom tIntercom)
    {
        List<TIntercom> list = tIntercomService.selectTIntercomList(tIntercom);
        ExcelUtil<TIntercom> util = new ExcelUtil<TIntercom>(TIntercom.class);
        return util.exportExcel(list, "intercom");
    }

    /**
     * 获取对讲机管理详细信息
     */
    @PreAuthorize("@ss.hasPermi('affair:intercom:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return AjaxResult.success(tIntercomService.selectTIntercomById(id));
    }

    /**
     * 新增对讲机管理
     */
    @PreAuthorize("@ss.hasPermi('affair:intercom:add')")
    @Log(title = "对讲机管理", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody TIntercom tIntercom)
    {
        tIntercom.setCreaterCode(getUserId().toString());
        return toAjax(tIntercomService.insertTIntercom(tIntercom));
    }

    /**
     * 修改对讲机管理
     */
    @PreAuthorize("@ss.hasPermi('affair:intercom:edit')")
    @Log(title = "对讲机管理", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody TIntercom tIntercom)
    {
        tIntercom.setUpdaterCode(getUserId().toString());
        tIntercom.setUpdatedate(new Date());
        return toAjax(tIntercomService.updateTIntercom(tIntercom));
    }

    /**
     * 删除对讲机管理
     */
    @PreAuthorize("@ss.hasPermi('affair:intercom:remove')")
    @Log(title = "对讲机管理", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(tIntercomService.deleteTIntercomByIds(ids));
    }

    /**
     * 批量导入对讲机管理
     */
    @PreAuthorize("@ss.hasPermi('affair:intercom:add')")
    @Log(title = "对讲机管理", businessType = BusinessType.INSERT)
    @PostMapping("/importData")
    public AjaxResult importData(@RequestParam("file") MultipartFile file) throws IOException
    {
        //获取操作人员ID
        Long userId = getUserId();
        Workbook workbook = ExcelUtils.getWorkBook(file);
        Sheet sheet = workbook.getSheetAt(0);
        List<TIntercom> list = new ArrayList<TIntercom>();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        //字典查询
        List<SysDictData> plant = iSysDictTypeService.selectDictDataByType("PLANT_DIVIDE");
        //部门查询
        List<SysDept> dept = iSysDeptService.selectDeptList(new SysDept());
        int rowNum = sheet.getPhysicalNumberOfRows();
        int failNumber = 0;
        //报错行数统计
        List<Integer> failRow =new ArrayList<Integer>();
        for (int i = 1; i < rowNum; i++) {
            try {
                logger.info("读取行数：" + i);
                Row row = sheet.getRow(i);
                int cellNum = row.getLastCellNum();
                TIntercom entity = new TIntercom();
                for (int j = 0; j < cellNum; j++) {
                    Cell cell = row.getCell(j);
                    //  cell.setCellType(CellType.STRING);
                    String cellValue = ExcelUtils.getCellValue(cell);
                    logger.info("cellValue:" + cellValue);
                    if (j == 0) {
                        for (SysDictData p : plant) {
                            if (p.getDictLabel().equals(cellValue.trim())) {
                                entity.setPlantCode(p.getDictValue());//装置名称
                            }
                        }
                    } else if (j == 1) {
                        if (cellValue != ""){
                            entity.setSerialnumber(Long.parseLong(cellValue));//序号
                        }
                    } else if (j == 2) {
                        entity.setDevno(cellValue);//设备编号
                    } else if (j == 3) {
                        entity.setDevtype(cellValue);//设备型号
                    } else if (j == 4) {
                        entity.setRecvdept(cellValue);//领用部门
                    } else if (j == 5) {
                        entity.setFixedassetsno(cellValue);//固定资产编号
                    } else if (j == 6) {
                        entity.setSerialno(cellValue);//S/N
                    } else if (j == 7) {
                        entity.setCostcenter(cellValue);//资产所属成本中心
                    } else if (j == 8) {
                        entity.setReceiver(cellValue);//领用人
                    } else if (j == 9) {
                        entity.setEuno(cellValue);//EU编号
                    } else if (j == 10) {
                        for (SysDept d : dept) {
                            if (d.getDeptName().equals(cellValue.trim())) {
                                entity.setDeptId(d.getDeptId());//部门编号
                            }
                        }
                    } else if (j == 11) {
                        entity.setRemarks(cellValue);//备注
                    }
                }
                entity.setCreaterCode(userId.toString());
                logger.info("entity:" + entity);
                list.add(entity);
            }catch (Exception e){
                failNumber++;
                failRow.add(i+1);
            }
        }
        int successNumber = 0;
        int failNum = 0;
        for (TIntercom t : list
        ) {
            failNum++;
            try {
                tIntercomService.insertTIntercom(t);
                successNumber++;
            }catch (Exception e){
                failNumber++;
                failRow.add(failNum+1);
            }
        }
        logger.info("list:" + JSON.toJSONString(list));
        logger.info("successNumber:" +String.valueOf(successNumber));
        logger.info("failNumber:" +String.valueOf(failNumber));
        logger.info("failRow:" +String.valueOf(failRow));
        return AjaxResult.success(String.valueOf(successNumber), failRow);
    }
}
