package com.ruoyi.project.ehs.controller;

import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import com.alibaba.fastjson.JSON;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.file.ExcelUtils;
import com.ruoyi.common.utils.file.FileUploadUtils;
import com.ruoyi.framework.config.RuoYiConfig;
import com.ruoyi.project.ehs.domain.TEnvironapproval;
import com.ruoyi.project.plant.domain.TStaffmgr;
import com.ruoyi.project.plant.service.ITStaffmgrService;
import com.ruoyi.project.system.domain.SysDept;
import com.ruoyi.project.system.domain.SysDictData;
import com.ruoyi.project.system.service.ISysDeptService;
import com.ruoyi.project.system.service.ISysDictTypeService;
import org.apache.poi.ss.usermodel.*;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import com.ruoyi.framework.aspectj.lang.annotation.Log;
import com.ruoyi.framework.aspectj.lang.enums.BusinessType;
import com.ruoyi.project.ehs.domain.TSafetyapproval;
import com.ruoyi.project.ehs.service.ITSafetyapprovalService;
import com.ruoyi.framework.web.controller.BaseController;
import com.ruoyi.framework.web.domain.AjaxResult;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.framework.web.page.TableDataInfo;
import org.springframework.web.multipart.MultipartFile;

/**
 * 安全批文清单Controller
 *
 * @author ruoyi
 * @date 2020-11-26
 */
@RestController
@RequestMapping("/ehs/safetyapproval")
public class TSafetyapprovalController extends BaseController
{
    @Autowired
    private ITSafetyapprovalService tSafetyapprovalService;

    @Autowired
    private ISysDeptService iSysDeptService;

    @Autowired
    private ISysDictTypeService iSysDictTypeService;

    @Autowired
    private ITStaffmgrService tStaffmgrService;

    /**
     * 查询安全批文清单列表
     */
    @PreAuthorize("@ss.hasPermi('ehs:safetyapproval:list')")
    @GetMapping("/list")
    public TableDataInfo list(TSafetyapproval tSafetyapproval)
    {
        startPage();
        List<TSafetyapproval> list = tSafetyapprovalService.selectTSafetyapprovalList(tSafetyapproval);
        return getDataTable(list);
    }

    /**
     * 导出安全批文清单列表
     */
    @PreAuthorize("@ss.hasPermi('ehs:safetyapproval:export')")
    @Log(title = "安全批文清单", businessType = BusinessType.EXPORT)
    @GetMapping("/export")
    public AjaxResult export(TSafetyapproval tSafetyapproval)
    {
        List<TSafetyapproval> list = tSafetyapprovalService.selectTSafetyapprovalList(tSafetyapproval);
        ExcelUtil<TSafetyapproval> util = new ExcelUtil<TSafetyapproval>(TSafetyapproval.class);
        return util.exportExcel(list, "safetyapproval");
    }

    /**
     * 获取安全批文清单详细信息
     */
    @PreAuthorize("@ss.hasPermi('ehs:safetyapproval:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return AjaxResult.success(tSafetyapprovalService.selectTSafetyapprovalById(id));
    }

    /**
     * 新增安全批文清单
     */
    @PreAuthorize("@ss.hasPermi('ehs:safetyapproval:add')")
    @Log(title = "安全批文清单", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody TSafetyapproval tSafetyapproval)
    {
        tSafetyapproval.setCreaterCode(getUserId().toString());
        return toAjax(tSafetyapprovalService.insertTSafetyapproval(tSafetyapproval));
    }

    /**
     * 修改安全批文清单
     */
    @PreAuthorize("@ss.hasPermi('ehs:safetyapproval:edit')")
    @Log(title = "安全批文清单", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody TSafetyapproval tSafetyapproval)
    {
        tSafetyapproval.setUpdaterCode(getUserId().toString());
        tSafetyapproval.setUpdatedate(new Date());
        return toAjax(tSafetyapprovalService.updateTSafetyapproval(tSafetyapproval));
    }

    /**
     * 删除安全批文清单
     */
    @PreAuthorize("@ss.hasPermi('ehs:safetyapproval:remove')")
    @Log(title = "安全批文清单", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(tSafetyapprovalService.deleteTSafetyapprovalByIds(ids));
    }

    /**
     * 批量导入安全批文清单
     */
    @PreAuthorize("@ss.hasPermi('ehs:safetyapproval:add')")
    @Log(title = "安全批文清单", businessType = BusinessType.INSERT)
    @PostMapping("/importData")
    public AjaxResult importData(@RequestParam("file") MultipartFile file) throws IOException
    {
        //获取操作人员ID
        Long userId = getUserId();
        //报错行数统计
        List<Integer> failRow =new ArrayList<Integer>();
        Workbook workbook = ExcelUtils.getWorkBook(file);
        Sheet sheet = workbook.getSheetAt(0);
        List<TSafetyapproval> list = new ArrayList<TSafetyapproval>();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        //字典查询
        List<SysDictData> plant = iSysDictTypeService.selectDictDataByType("PLANT_DIVIDE");
        List<SysDictData> yesNo = iSysDictTypeService.selectDictDataByType("YES_NO");
        //部门查询
        List<SysDept> dept = iSysDeptService.selectDeptList(new SysDept());
        //人员查询
        List<TStaffmgr> staffmgrs = tStaffmgrService.selectTStaffmgrList(new TStaffmgr());
        int rowNum = sheet.getPhysicalNumberOfRows();
        int failNumber = 0;
        for (int i = 1; i < rowNum; i++) {
            try {
                logger.info("读取行数：" + i);
                Row row = sheet.getRow(i);
                int cellNum = row.getLastCellNum();
                TSafetyapproval entity = new TSafetyapproval();
                for (int j = 0; j < cellNum; j++) {
                    Cell cell = row.getCell(j);
                    //  cell.setCellType(CellType.STRING);
                    String cellValue = ExcelUtils.getCellValue(cell);
                    logger.info("cellValue:" + cellValue);
                    if (j == 0) {
                        for (SysDictData p : plant) {
                            if (p.getDictLabel().equals(cellValue.trim())) {
                                entity.setPlantCode(p.getDictValue());//装置名称
                            }
                        }
                    } else if (j == 1) {
                        entity.setApprovalname(cellValue);//批文名称
                    } else if (j == 2) {
                        entity.setCertno(cellValue);//证书编号
                    } else if (j == 3) {
                        if (cellValue.length() > 3) {
                            entity.setEffetivedate(new SimpleDateFormat(DateUtils.getDateFormat(cellValue)).parse(cellValue));//批准日期
                        }
                    } else if (j == 4) {
                        entity.setScope(cellValue);//适用范围
                    } else if (j == 5) {
                        entity.setRelatedlaw(cellValue);//相关法规
                    } else if (j == 6) {
                        entity.setResponsauth(cellValue);//审批单位
                    } else if (j == 7) {
                        if (cellValue.length() > 3) {
                            entity.setValidityBefore(new SimpleDateFormat(DateUtils.getDateFormat(cellValue)).parse(cellValue));//证书有效期前
                            entity.setIsPermanent(Long.parseLong("0"));
                        }else {
                            entity.setIsPermanent(Long.parseLong("1"));
                        }
                    } else if (j == 8) {
                        if (cellValue.length() > 3) {
                            entity.setValidityAfter(new SimpleDateFormat(DateUtils.getDateFormat(cellValue)).parse(cellValue));//证书有效期后
                        }
                    } else if (j == 9) {
                        for (SysDictData p : yesNo) {
                            if (p.getDictLabel().equals(cellValue.trim())) {
                                entity.setFollow(p.getDictValue());//是否需要跟进
                            }
                        }
                    } else if (j == 10) {
                        entity.setAllowance(cellValue);//使用/储存/废物最大量
                    } else if (j == 11) {
                        entity.setRequirements(cellValue);//排放/监控要求
                    } else if (j == 12) {
                        entity.setContent(cellValue);//主要内容
                    } else if (j == 13) {
                        entity.setOwner(cellValue);//负责人
                    } else if (j == 15) {
                        for (TStaffmgr s : staffmgrs) {
                            if (s.getStaffid().equals(cellValue.trim())) {
                                entity.setReviewer(s.getStaffid());//回顾人
                            }
                        }
                    } else if (j == 16) {
                        if (cellValue.length() > 3) {
                            entity.setReviewdate(new SimpleDateFormat(DateUtils.getDateFormat(cellValue)).parse(cellValue));//本次回顾日期
                        }
                    } else if (j == 17) {
                        entity.setReviewResult(cellValue);//回顾结果
                    } else if (j == 18) {
                        if (cellValue.length() > 3) {
                            entity.setNextreviewdate(new SimpleDateFormat(DateUtils.getDateFormat(cellValue)).parse(cellValue));//下次回顾日期
                        }
                    } else if (j == 19) {
                        for (SysDept d : dept) {
                            if (d.getDeptName().equals(cellValue.trim())) {
                                entity.setDeptId(d.getDeptId());//部门编号
                            }
                        }
                    } else if (j == 20) {
                        entity.setRemarks(cellValue);//备注
                    }
                }
                entity.setCreaterCode(userId.toString());
                logger.info("entity:" + entity);
                list.add(entity);
            }catch (Exception e){
                failNumber++;
                failRow.add(i+1);
            }
        }
        int successNumber = 0;
        int failNum = 0;
        for (TSafetyapproval t : list
        ) {
            failNum++;
            try {
                tSafetyapprovalService.insertTSafetyapproval(t);
                successNumber++;
            }catch (Exception e){
                failNumber++;
                failRow.add(failNum+1);
            }
        }
        logger.info("list:" + JSON.toJSONString(list));
        logger.info("successNumber:" +String.valueOf(successNumber));
        logger.info("failNumber:" +String.valueOf(failNumber));
        logger.info("failRow:" +String.valueOf(failRow));
        return AjaxResult.success(String.valueOf(successNumber), failRow);
    }

    /**
     * 图片上传
     */
    @PostMapping("/uploadPic")
    public AjaxResult uploadFile(@RequestParam("file") MultipartFile file,String pType,String pId) throws IOException
    {
        if (!file.isEmpty())
        {
            String url = FileUploadUtils.upload(RuoYiConfig.getFilePath("/"+ pType), file);
            return AjaxResult.success(url);
        }
        return AjaxResult.error("上传失败，请联系管理员");
    }
}
