package com.ruoyi.project.intact.controller;

import java.util.List;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.framework.aspectj.lang.annotation.Log;
import com.ruoyi.framework.aspectj.lang.enums.BusinessType;
import com.ruoyi.project.intact.domain.TIntactGj;
import com.ruoyi.project.intact.service.ITIntactGjService;
import com.ruoyi.framework.web.controller.BaseController;
import com.ruoyi.framework.web.domain.AjaxResult;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.framework.web.page.TableDataInfo;

/**
 * 设备完整性管件台账Controller
 *
 * @author ruoyi
 * @date 2022-06-22
 */
@RestController
@RequestMapping("/intact/gj")
public class TIntactGjController extends BaseController
{
    @Autowired
    private ITIntactGjService tIntactGjService;

    /**
     * 查询设备完整性管件台账列表
     */
    @PreAuthorize("@ss.hasPermi('intact:gj:list')")
    @GetMapping("/list")
    public TableDataInfo list(TIntactGj tIntactGj)
    {
        startPage();
        List<TIntactGj> list = tIntactGjService.selectTIntactGjList(tIntactGj);
        return getDataTable(list);
    }

    /**
     * 导出设备完整性管件台账列表
     */
    @PreAuthorize("@ss.hasPermi('intact:gj:export')")
    @Log(title = "设备完整性管件台账", businessType = BusinessType.EXPORT)
    @GetMapping("/export")
    public AjaxResult export(TIntactGj tIntactGj)
    {
        List<TIntactGj> list = tIntactGjService.selectTIntactGjList(tIntactGj);
        ExcelUtil<TIntactGj> util = new ExcelUtil<TIntactGj>(TIntactGj.class);
        return util.exportExcel(list, "gj");
    }

    /**
     * 获取设备完整性管件台账详细信息
     */
    @PreAuthorize("@ss.hasPermi('intact:gj:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return AjaxResult.success(tIntactGjService.selectTIntactGjById(id));
    }

    /**
     * 新增设备完整性管件台账
     */
    @PreAuthorize("@ss.hasPermi('intact:gj:add')")
    @Log(title = "设备完整性管件台账", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody TIntactGj tIntactGj)
    {
        return toAjax(tIntactGjService.insertTIntactGj(tIntactGj));
    }

    /**
     * 修改设备完整性管件台账
     */
    @PreAuthorize("@ss.hasPermi('intact:gj:edit')")
    @Log(title = "设备完整性管件台账", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody TIntactGj tIntactGj)
    {
        return toAjax(tIntactGjService.updateTIntactGj(tIntactGj));
    }

    /**
     * 删除设备完整性管件台账
     */
    @PreAuthorize("@ss.hasPermi('intact:gj:remove')")
    @Log(title = "设备完整性管件台账", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(tIntactGjService.deleteTIntactGjByIds(ids));
    }
}
