package com.ruoyi.project.patrol.commonuse.controller;

import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.file.ExcelUtils;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.framework.aspectj.lang.annotation.Log;
import com.ruoyi.framework.aspectj.lang.enums.BusinessType;
import com.ruoyi.framework.interceptor.annotation.RepeatSubmit;
import com.ruoyi.framework.web.controller.BaseController;
import com.ruoyi.framework.web.domain.AjaxResult;
import com.ruoyi.framework.web.page.TableDataInfo;
import com.ruoyi.project.patrol.commonuse.domain.TCommonuseWalkieTalkie;
import com.ruoyi.project.patrol.commonuse.service.ITCommonuseWalkieTalkieService;
import com.ruoyi.project.patrol.patrol.domain.TPatrolIssues;
import com.ruoyi.project.patrol.patrol.domain.TPatrolPlan;
import com.ruoyi.project.patrol.patrol.service.ITPatrolIssuesService;
import com.ruoyi.project.patrol.patrol.service.ITPatrolPlanService;
import com.ruoyi.project.system.domain.SysUser;
import com.ruoyi.project.system.service.ISysUserService;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.ss.util.CellRangeAddress;
import org.apache.poi.xssf.usermodel.XSSFCell;
import org.apache.poi.xssf.usermodel.XSSFRow;
import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import java.io.IOException;
import java.io.OutputStream;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.Objects;

/**
 * 对讲机Controller
 *
 * @author admin
 * @date 2023-11-08
 */
@RestController
@RequestMapping("/commonuse/walkitTalkie")
public class TCommonuseWalkieTalkieController extends BaseController {
    @Autowired
    private ITCommonuseWalkieTalkieService tCommonuseWalkieTalkieService;

    @Autowired
    private ITPatrolPlanService tPatrolPlanService;

    @Autowired
    private ITPatrolIssuesService tPatrolIssuesService;

    @Autowired
    private ISysUserService sysUserService;

    /**
     * 查询对讲机列表
     */
    @PreAuthorize("@ss.hasPermi('commonuse:walkitTalkie:list')")
    @GetMapping("/list")
    public TableDataInfo list(TCommonuseWalkieTalkie tCommonuseWalkieTalkie) {
        startPage();
        if (StringUtils.isEmpty(tCommonuseWalkieTalkie.getIssuesFlag()))
            tCommonuseWalkieTalkie.setIssuesFlag("0");
        List<TCommonuseWalkieTalkie> list = tCommonuseWalkieTalkieService.selectTCommonuseWalkieTalkieList(tCommonuseWalkieTalkie);
        return getDataTable(list);
    }

    /**
     * 导出对讲机列表
     */
    
    @Log(title = "对讲机", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public AjaxResult export(TCommonuseWalkieTalkie tCommonuseWalkieTalkie) {
        try {
            tCommonuseWalkieTalkie.setIssuesFlag("0");
            List<TCommonuseWalkieTalkie> list = tCommonuseWalkieTalkieService.selectTCommonuseWalkieTalkieList(tCommonuseWalkieTalkie);
            XSSFWorkbook workbook = new XSSFWorkbook();
            // 创建工作簿
            XSSFSheet sheet = workbook.createSheet();
            // 设置列宽
            sheet.setColumnWidth(0, 1200);
            sheet.setColumnWidth(1, 3500);
            sheet.setColumnWidth(2, 3500);
            sheet.setColumnWidth(3, 4500);
            sheet.setColumnWidth(4, 4500);
            sheet.setColumnWidth(5, 5500);
            sheet.setColumnWidth(6, 4500);
            sheet.setColumnWidth(7, 4500);
            //第一行
            XSSFRow row1 = sheet.createRow(0);
            row1.createCell(0).setCellValue("CBP-6.3.0-SCFSH-020.26  Rev:04  2021/10/01");
            ExcelUtils.createVoidCell(1, 7, row1);
            sheet.addMergedRegion(new CellRangeAddress(0, 0, 0, 7));//合并
            //第二行
            XSSFRow row2 = sheet.createRow(1);
            row2.createCell(0).setCellValue("对讲机检查记录");
            ExcelUtils.createVoidCell(1, 7, row2);
            sheet.addMergedRegion(new CellRangeAddress(1, 1, 0, 7));
            //第三行
            XSSFRow row3 = sheet.createRow(2);
            row3.createCell(0).setCellValue("年度：" + tCommonuseWalkieTalkie.getYear());
            sheet.addMergedRegion(new CellRangeAddress(2, 2, 0, 1));
            ExcelUtils.createVoidCell(1, 7, row3);
            row3.createCell(2).setCellValue("检查频率：1次/月");
            sheet.addMergedRegion(new CellRangeAddress(2, 2, 2, 7));
            //第四行
            XSSFRow row4 = sheet.createRow(3);
            row4.createCell(0).setCellValue("序号");
            row4.createCell(1).setCellValue("位号");
            row4.createCell(2).setCellValue("检查内容");
            ExcelUtils.createVoidCell(4, 7, row4);
            sheet.addMergedRegion(new CellRangeAddress(3, 3, 2, 7));
            //第五行
            XSSFRow row5 = sheet.createRow(4);
            sheet.addMergedRegion(new CellRangeAddress(3, 4, 0, 0));
            sheet.addMergedRegion(new CellRangeAddress(3, 4, 1, 1));
            ExcelUtils.createVoidCell(0, 2, row5);
            row5.createCell(2).setCellValue("外观情况");
            row5.createCell(3).setCellValue("编号标签");
            row5.createCell(4).setCellValue("电池状况");
            row5.createCell(5).setCellValue("显示屏状况");
            row5.createCell(6).setCellValue("通话音质");
            row5.createCell(7).setCellValue("检查人");
            Long planId = null;
            //遍历数组赋值
            for (int i = 0; i < list.size(); i++) {
                TCommonuseWalkieTalkie talkie = list.get(i);
                planId = talkie.getPlanId();
                XSSFRow row = sheet.createRow(5 + i);
                row.createCell(0).setCellValue(i + 1);
                row.createCell(1).setCellValue(talkie.getDevNo());
                row.createCell(2).setCellValue("1".equals(talkie.getAppearance()) ? "√" : talkie.getAppearance());
                row.createCell(3).setCellValue("1".equals(talkie.getNumberingLabel()) ? "√" : talkie.getNumberingLabel());
                row.createCell(4).setCellValue("1".equals(talkie.getBatteryCondition()) ? "√" : talkie.getBatteryCondition());
                row.createCell(5).setCellValue("1".equals(talkie.getDisplayScreen()) ? "√" : talkie.getDisplayScreen());
                row.createCell(6).setCellValue("1".equals(talkie.getVoiceQuality()) ? "√" : talkie.getVoiceQuality());
                XSSFCell cell10 = row.createCell(7);
                if (StringUtils.isNotEmpty(talkie.getUpdaterCode())) {
                    SysUser sysUser = sysUserService.selectUserById(Long.valueOf(talkie.getUpdaterCode()));
                    cell10.setCellValue(sysUser == null ? "" : sysUser.getNickName());
                }
            }
            String confirmed = "";
            Date confirmedDate = null;
            TPatrolPlan tPatrolPlan = tPatrolPlanService.selectTPatrolPlanById(planId);
            if (Objects.nonNull(tPatrolPlan) && Objects.nonNull(tPatrolPlan.getConfirmed())) {
                SysUser sysUser = sysUserService.selectUserById(Long.valueOf(tPatrolPlan.getConfirmed()));            if (sysUser!=null) {
                confirmed = sysUser.getSignUrl();
                confirmedDate = tPatrolPlan.getConfirmDate();
            }
            }
            // 循环后下一行
            int nextRowNum = 5 + list.size();
            XSSFRow row28 = sheet.createRow(nextRowNum);
            row28.setHeightInPoints(30);
            XSSFCell row28Cell1 = row28.createCell(0);
            row28Cell1.setCellValue("签名");
            ExcelUtils.createVoidCell(1, 2, row28);
            sheet.addMergedRegion(new CellRangeAddress(nextRowNum, nextRowNum, 0, 1));
            if (StringUtils.isNotEmpty(confirmed))
                ExcelUtils.insertPicture(workbook, sheet, confirmed, nextRowNum, 5, 1, 1);
            ExcelUtils.createVoidCell(2, 7, row28);
            sheet.addMergedRegion(new CellRangeAddress(nextRowNum, nextRowNum, 2, 7));
            // 下一行
            nextRowNum++;
            XSSFRow row29 = sheet.createRow(nextRowNum);
            row29.setHeightInPoints(30);
            XSSFCell row29Cell1 = row29.createCell(0);
            row29Cell1.setCellValue("日期");
            ExcelUtils.createVoidCell(1, 1, row29);
            sheet.addMergedRegion(new CellRangeAddress(nextRowNum, nextRowNum, 0, 1));
            row29.createCell(2).setCellValue(confirmedDate != null ? new SimpleDateFormat("yyyy-MM-dd").format(confirmedDate) : "");
            ExcelUtils.createVoidCell(3, 7, row29);
            sheet.addMergedRegion(new CellRangeAddress(nextRowNum, nextRowNum, 2, 7));
            // 设置单元格样式
            // 文字居中，自动换行，四周边框
            CellStyle style = workbook.createCellStyle();
            style.setAlignment(HorizontalAlignment.CENTER);
            style.setVerticalAlignment(VerticalAlignment.CENTER);
            style.setWrapText(true);
            style.setBorderTop(BorderStyle.THIN);
            style.setBorderBottom(BorderStyle.THIN);
            style.setBorderLeft(BorderStyle.THIN);
            style.setBorderRight(BorderStyle.THIN);

            // 设置垂直居中，四周边框
            CellStyle style2 = workbook.createCellStyle();
            style2.setVerticalAlignment(VerticalAlignment.CENTER);
            style2.setBorderTop(BorderStyle.THIN);
            style2.setBorderBottom(BorderStyle.THIN);
            style2.setBorderLeft(BorderStyle.THIN);
            style2.setBorderRight(BorderStyle.THIN);
            for (Row row : sheet) {
                row.setHeightInPoints(30);
                if (row.getRowNum() == 0) {
                    continue;
                }
                if (row.getRowNum() == 2) {
                    for (Cell cell : row) {
                        if (cell.getColumnIndex() == 0 || cell.getColumnIndex() == 1)
                            cell.setCellStyle(style2);
                        else
                            cell.setCellStyle(style);
                    }
                } else {
                    for (Cell cell : row) {
                        cell.setCellStyle(style);
                    }
                }
            }
            nextRowNum++;
            XSSFRow row30 = sheet.createRow(nextRowNum);
            row30.createCell(0).setCellValue("说明：");
            ExcelUtils.createVoidCell(1, 7, row30);
            sheet.addMergedRegion(new CellRangeAddress(nextRowNum, nextRowNum, 0, 7));

            nextRowNum++;
            XSSFRow row31 = sheet.createRow(nextRowNum);
            row31.createCell(0).setCellValue("1.检查正常打“√”。");
            ExcelUtils.createVoidCell(1, 7, row31);
            sheet.addMergedRegion(new CellRangeAddress(nextRowNum, nextRowNum, 0, 7));

            nextRowNum++;
            XSSFRow row32 = sheet.createRow(nextRowNum);
            row32.createCell(0).setCellValue("2.检查有不正常情况打“X”，并记录存在问题，处理结果及完成时间。");
            ExcelUtils.createVoidCell(1, 7, row32);
            sheet.addMergedRegion(new CellRangeAddress(nextRowNum, nextRowNum, 0, 7));

            nextRowNum++;
            XSSFRow row33 = sheet.createRow(nextRowNum);
            row33.createCell(0).setCellValue("3. 检查频率为每一个月检查维护一次。");
            ExcelUtils.createVoidCell(1, 7, row33);
            sheet.addMergedRegion(new CellRangeAddress(nextRowNum, nextRowNum, 0, 7));

            //返回
            String fileName = ExcelUtil.encodingFilename("main");
            OutputStream os = Files.newOutputStream(Paths.get(ExcelUtil.getAbsoluteFile(fileName)));
            workbook.write(os);
            workbook.close();
            return AjaxResult.success(fileName);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 获取对讲机详细信息
     */
    @PreAuthorize("@ss.hasPermi('commonuse:walkitTalkie:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id) {
        return AjaxResult.success(tCommonuseWalkieTalkieService.selectTCommonuseWalkieTalkieById(id));
    }

    /**
     * 获取对讲机详细信息
     */
    @PreAuthorize("@ss.hasPermi('commonuse:walkitTalkie:query')")
    @GetMapping(value = "/info")
    public AjaxResult getInfo( TCommonuseWalkieTalkie tCommonuseWalkieTalkie) {
        if (StringUtils.isNull(tCommonuseWalkieTalkie.getDevNo()) || StringUtils.isNull(tCommonuseWalkieTalkie.getPlanId())) {
            return AjaxResult.error("位号或计划id不能为空！");
        }
        TCommonuseWalkieTalkie item = new TCommonuseWalkieTalkie();
        item.setPlanId(tCommonuseWalkieTalkie.getPlanId());
        item.setPrecision(tCommonuseWalkieTalkie.getDevNo());
        item.setIssuesFlag("0");
        List<TCommonuseWalkieTalkie> list = tCommonuseWalkieTalkieService.selectTCommonuseWalkieTalkieList(item);
        if (CollectionUtils.isNotEmpty(list))
            return AjaxResult.success(list.get(0));
        return AjaxResult.error("此设备不在当前巡检计划中!");
    }

    /**
     * 新增对讲机
     */
    @PreAuthorize("@ss.hasPermi('commonuse:walkitTalkie:add')")
    @Log(title = "对讲机", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody TCommonuseWalkieTalkie tCommonuseWalkieTalkie) {
        return toAjax(tCommonuseWalkieTalkieService.insertTCommonuseWalkieTalkie(tCommonuseWalkieTalkie));
    }

    /**
     * 修改对讲机
     */
    @PreAuthorize("@ss.hasPermi('commonuse:walkitTalkie:edit')")
    @Log(title = "对讲机", businessType = BusinessType.UPDATE)
    @PutMapping
    @RepeatSubmit
    public AjaxResult edit(@RequestBody TCommonuseWalkieTalkie tCommonuseWalkieTalkie) {
        tCommonuseWalkieTalkie.setUpdatedate(new Date());
        tCommonuseWalkieTalkie.setUpdaterCode(getUserId().toString());
        tCommonuseWalkieTalkie.setCheckStatus("1");
        tCommonuseWalkieTalkie.setCheckDate(new Date());
        TCommonuseWalkieTalkie item = tCommonuseWalkieTalkieService.selectTCommonuseWalkieTalkieById(tCommonuseWalkieTalkie.getId());
        TPatrolPlan tPatrolPlan = tPatrolPlanService.selectTPatrolPlanById(tCommonuseWalkieTalkie.getPlanId());
        if (tPatrolPlan.getStatus()==1){
            return AjaxResult.error("当前选择的计划已确认！");
        }
        if (!"1".equals(item.getCheckStatus())) {
            tPatrolPlan.setUncheckedNum(tPatrolPlan.getUncheckedNum() - 1);
            tPatrolPlan.setCheckedNum(tPatrolPlan.getCheckedNum() + 1);
            tPatrolPlanService.updateTPatrolPlan(tPatrolPlan);
        }
        if (StringUtils.isNotEquals(tCommonuseWalkieTalkie.getAppearance(), "1")
                || StringUtils.isNotEquals(tCommonuseWalkieTalkie.getBatteryCondition(), "1")
                || StringUtils.isNotEquals(tCommonuseWalkieTalkie.getDisplayScreen(), "1")
                || StringUtils.isNotEquals(tCommonuseWalkieTalkie.getVoiceQuality(), "1")
                || StringUtils.isNotEquals(tCommonuseWalkieTalkie.getNumberingLabel(), "1")) {
            tCommonuseWalkieTalkie.setIssuesStatus("1");
            tCommonuseWalkieTalkie.setCheckStatus("0");
            tCommonuseWalkieTalkieService.updateTCommonuseWalkieTalkie(tCommonuseWalkieTalkie);
            tCommonuseWalkieTalkie.setIssuesFlag("1");
            tCommonuseWalkieTalkie.setCheckStatus("1");
            tCommonuseWalkieTalkie.setCreatedate(new Date());
            tCommonuseWalkieTalkie.setCreaterCode(getUserId().toString());
            tCommonuseWalkieTalkie.setDevNo(item.getDevNo());
            tCommonuseWalkieTalkieService.insertTCommonuseWalkieTalkie(tCommonuseWalkieTalkie);
            TPatrolIssues tPatrolIssues = new TPatrolIssues();
            tPatrolIssues.setPatrolType("23");
            tPatrolIssues.setDevDescribe(tCommonuseWalkieTalkie.getDevNo());
            tPatrolIssues.setLinkId(tCommonuseWalkieTalkie.getId());
            tPatrolIssues.setStatus(0L);
            tPatrolIssuesService.insertTPatrolIssues(tPatrolIssues);
        } else {
            if ("1".equals(item.getIssuesStatus())) {
                TPatrolIssues tPatrolIssues = new TPatrolIssues();
                tPatrolIssues.setPatrolType("23");
                tPatrolIssues.setDevDescribe(item.getDevNo());
                tPatrolIssues.setCreatedate(new Date());
                for (TPatrolIssues patrolIssues : tPatrolIssuesService.selectTPatrolIssuesList(tPatrolIssues)) {
                    patrolIssues.setStatus(1L);
                    tPatrolIssuesService.updateTPatrolIssues(patrolIssues);
                }
            }
            tCommonuseWalkieTalkie.setIssuesStatus("0");
            tCommonuseWalkieTalkieService.updateTCommonuseWalkieTalkie(tCommonuseWalkieTalkie);
        }
        return AjaxResult.success();
    }

    /**
     * 删除对讲机
     */
    @PreAuthorize("@ss.hasPermi('commonuse:walkitTalkie:remove')")
    @Log(title = "对讲机", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids) {
        return toAjax(tCommonuseWalkieTalkieService.deleteTCommonuseWalkieTalkieByIds(ids));
    }
}
