package com.ruoyi.project.patrol.fire.controller;

import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.file.ExcelUtils;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.framework.aspectj.lang.annotation.Log;
import com.ruoyi.framework.aspectj.lang.enums.BusinessType;
import com.ruoyi.framework.interceptor.annotation.RepeatSubmit;
import com.ruoyi.framework.web.controller.BaseController;
import com.ruoyi.framework.web.domain.AjaxResult;
import com.ruoyi.framework.web.page.TableDataInfo;
import com.ruoyi.project.patrol.fire.domain.TFireGasShield;
import com.ruoyi.project.patrol.fire.service.ITFireGasShieldService;
import com.ruoyi.project.patrol.patrol.domain.TPatrolIssues;
import com.ruoyi.project.patrol.patrol.domain.TPatrolPlan;
import com.ruoyi.project.patrol.patrol.service.ITPatrolIssuesService;
import com.ruoyi.project.patrol.patrol.service.ITPatrolPlanService;
import com.ruoyi.project.system.domain.SysUser;
import com.ruoyi.project.system.service.ISysUserService;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.ss.util.CellRangeAddress;
import org.apache.poi.xssf.usermodel.XSSFCell;
import org.apache.poi.xssf.usermodel.XSSFRow;
import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import java.io.IOException;
import java.io.OutputStream;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.Objects;
import java.util.UUID;

/**
 * CO2气体保护系统Controller
 *
 * @author admin
 * @date 2023-11-01
 */
@RestController
@RequestMapping("/fire/shield")
public class TFireGasShieldController extends BaseController {
    @Autowired
    private ITFireGasShieldService tFireGasShieldService;

    @Autowired
    private ITPatrolPlanService tPatrolPlanService;

    @Autowired
    private ITPatrolIssuesService tPatrolIssuesService;

    @Autowired
    private ISysUserService sysUserService;

    /**
     * 查询CO2气体保护系统列表
     */
    @PreAuthorize("@ss.hasPermi('fire:shield:list')")
    @GetMapping("/list")
    public TableDataInfo list(TFireGasShield tFireGasShield) {
        startPage();
        if (StringUtils.isEmpty(tFireGasShield.getIssuesFlag()))
            tFireGasShield.setIssuesFlag("0");
        List<TFireGasShield> list = tFireGasShieldService.selectTFireGasShieldList(tFireGasShield);
        return getDataTable(list);
    }

    /**
     * 导出CO2气体保护系统列表
     */
    
    @Log(title = "CO2气体保护系统", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public AjaxResult export(TFireGasShield tFireGasShield) {
        try {
            tFireGasShield.setIssuesFlag("0");
            List<TFireGasShield> list = tFireGasShieldService.selectTFireGasShieldList(tFireGasShield);
            XSSFWorkbook workbook = new XSSFWorkbook();
            // 创建工作簿
            XSSFSheet sheet = workbook.createSheet();
            // 设置列宽
            sheet.setColumnWidth(0, 1200);
            sheet.setColumnWidth(1, 3500);
            sheet.setColumnWidth(2, 3500);
            sheet.setColumnWidth(3, 4500);
            sheet.setColumnWidth(4, 4500);
            sheet.setColumnWidth(5, 5500);
            sheet.setColumnWidth(6, 4500);
            sheet.setColumnWidth(7, 4500);
            sheet.setColumnWidth(8, 4500);
            sheet.setColumnWidth(9, 4500);
            sheet.setColumnWidth(10, 4500);
            //第一行
            XSSFRow row1 = sheet.createRow(0);
            row1.createCell(0).setCellValue("CBP-6.3.0-SCFSH-020.11  Rev:04  2021/10/01");
            ExcelUtils.createVoidCell(1, 10, row1);
            sheet.addMergedRegion(new CellRangeAddress(0, 0, 0, 10));//合并
            //第二行
            XSSFRow row2 = sheet.createRow(1);
            row2.createCell(0).setCellValue("CO2气体保护系统检查记录");
            ExcelUtils.createVoidCell(1, 10, row2);
            sheet.addMergedRegion(new CellRangeAddress(1, 1, 0, 10));
            //第三行
            XSSFRow row3 = sheet.createRow(2);
            row3.createCell(0).setCellValue("年度：" + tFireGasShield.getYear());
            sheet.addMergedRegion(new CellRangeAddress(2, 2, 0, 1));
            ExcelUtils.createVoidCell(1, 10, row3);
            row3.createCell(2).setCellValue("检查频率：1次/月");
            sheet.addMergedRegion(new CellRangeAddress(2, 2, 2, 10));
            //第四行
            XSSFRow row4 = sheet.createRow(3);
            row4.createCell(0).setCellValue("序号");
            row4.createCell(1).setCellValue("位号");
            row4.createCell(2).setCellValue("检查内容");
            ExcelUtils.createVoidCell(4, 10, row4);
            sheet.addMergedRegion(new CellRangeAddress(3, 3, 2, 10));
            //第五行
            XSSFRow row5 = sheet.createRow(4);
            sheet.addMergedRegion(new CellRangeAddress(3, 4, 0, 0));
            sheet.addMergedRegion(new CellRangeAddress(3, 4, 1, 1));
            ExcelUtils.createVoidCell(0, 2, row5);
            row5.createCell(2).setCellValue("CO2灭火系统组件外观");
            row5.createCell(3).setCellValue("钢瓶间安全警示标志");
            row5.createCell(4).setCellValue("电仪机柜间气体释放警示标识");
            row5.createCell(5).setCellValue("钢瓶间操作卡片、标识");
            row5.createCell(6).setCellValue("钢瓶间的照明");
            row5.createCell(7).setCellValue("气动源的压力");
            row5.createCell(8).setCellValue("灭火控制盘工作状态及声光报警和放气指示灯工作状态");
            row5.createCell(9).setCellValue("秤重报警装置工作状态");
            row5.createCell(10).setCellValue("检查人");
            Long planId = null;
            //遍历数组赋值
            for (int i = 0; i < list.size(); i++) {
                TFireGasShield shield = list.get(i);
                planId = shield.getPlanId();
                XSSFRow row = sheet.createRow(5 + i);
                row.createCell(0).setCellValue(i + 1);
                row.createCell(1).setCellValue(shield.getDevNo());
                row.createCell(2).setCellValue("1".equals(shield.getComponentAppearance()) ? "√" : shield.getComponentAppearance());
                row.createCell(3).setCellValue("1".equals(shield.getSafetySign()) ? "√" : shield.getSafetySign());
                row.createCell(4).setCellValue("1".equals(shield.getReleaseSign()) ? "√" : shield.getReleaseSign());
                row.createCell(5).setCellValue("1".equals(shield.getCardSign()) ? "√" : shield.getCardSign());
                row.createCell(6).setCellValue("1".equals(shield.getIllumination()) ? "√" : shield.getIllumination());
                row.createCell(7).setCellValue("1".equals(shield.getPressure()) ? "√" : shield.getPressure());
                row.createCell(8).setCellValue("1".equals(shield.getIndicatorStatus()) ? "√" : shield.getIndicatorStatus());
                row.createCell(9).setCellValue("1".equals(shield.getDeviceStatus()) ? "√" : shield.getDeviceStatus());
                XSSFCell cell10 = row.createCell(10);
                if (StringUtils.isNotEmpty(shield.getUpdaterCode())) {
                    SysUser sysUser = sysUserService.selectUserById(Long.valueOf(shield.getUpdaterCode()));
                    cell10.setCellValue(sysUser == null ? "" : sysUser.getNickName());
                }
            }
            String confirmed = "";
            Date confirmedDate = null;
            TPatrolPlan tPatrolPlan = tPatrolPlanService.selectTPatrolPlanById(planId);
            if (Objects.nonNull(tPatrolPlan) && Objects.nonNull(tPatrolPlan.getConfirmed())) {
                SysUser sysUser = sysUserService.selectUserById(Long.valueOf(tPatrolPlan.getConfirmed()));            if (sysUser!=null) {
                confirmed = sysUser.getSignUrl();
                confirmedDate = tPatrolPlan.getConfirmDate();
            }
            }
            // 循环后下一行
            int nextRowNum = 5 + list.size();
            XSSFRow row28 = sheet.createRow(nextRowNum);
            row28.setHeightInPoints(30);
            XSSFCell row28Cell1 = row28.createCell(0);
            row28Cell1.setCellValue("签名");
            ExcelUtils.createVoidCell(1, 2, row28);
            sheet.addMergedRegion(new CellRangeAddress(nextRowNum, nextRowNum, 0, 1));
            if (StringUtils.isNotEmpty(confirmed))
                ExcelUtils.insertPicture(workbook, sheet, confirmed, nextRowNum, 6, 1, 1);
            ExcelUtils.createVoidCell(2, 10, row28);
            sheet.addMergedRegion(new CellRangeAddress(nextRowNum, nextRowNum, 2, 10));
            // 下一行
            nextRowNum++;
            XSSFRow row29 = sheet.createRow(nextRowNum);
            row29.setHeightInPoints(30);
            XSSFCell row29Cell1 = row29.createCell(0);
            row29Cell1.setCellValue("日期");
            ExcelUtils.createVoidCell(1, 1, row29);
            sheet.addMergedRegion(new CellRangeAddress(nextRowNum, nextRowNum, 0, 1));
            row29.createCell(2).setCellValue(confirmedDate != null ? new SimpleDateFormat("yyyy-MM-dd").format(confirmedDate) : "");
            ExcelUtils.createVoidCell(3, 10, row29);
            sheet.addMergedRegion(new CellRangeAddress(nextRowNum, nextRowNum, 2, 10));
            // 设置单元格样式
            // 文字居中，自动换行，四周边框
            CellStyle style = workbook.createCellStyle();
            style.setAlignment(HorizontalAlignment.CENTER);
            style.setVerticalAlignment(VerticalAlignment.CENTER);
            style.setWrapText(true);
            style.setBorderTop(BorderStyle.THIN);
            style.setBorderBottom(BorderStyle.THIN);
            style.setBorderLeft(BorderStyle.THIN);
            style.setBorderRight(BorderStyle.THIN);

            // 设置垂直居中，四周边框
            CellStyle style2 = workbook.createCellStyle();
            style2.setVerticalAlignment(VerticalAlignment.CENTER);
            style2.setBorderTop(BorderStyle.THIN);
            style2.setBorderBottom(BorderStyle.THIN);
            style2.setBorderLeft(BorderStyle.THIN);
            style2.setBorderRight(BorderStyle.THIN);
            for (Row row : sheet) {
                row.setHeightInPoints(30);
                if (row.getRowNum() == 0) {
                    continue;
                }
                if (row.getRowNum() == 4) {

                    row.setHeightInPoints(70);
                }
                if (row.getRowNum() == 2) {
                    for (Cell cell : row) {
                        if (cell.getColumnIndex() == 0 || cell.getColumnIndex() == 1)
                            cell.setCellStyle(style2);
                        else
                            cell.setCellStyle(style);
                    }
                } else {
                    for (Cell cell : row) {
                        cell.setCellStyle(style);
                    }
                }
            }
            nextRowNum++;
            XSSFRow row30 = sheet.createRow(nextRowNum);
            row30.createCell(0).setCellValue("说明：");
            ExcelUtils.createVoidCell(1, 10, row30);
            sheet.addMergedRegion(new CellRangeAddress(nextRowNum, nextRowNum, 0, 10));

            nextRowNum++;
            XSSFRow row31 = sheet.createRow(nextRowNum);
            row31.createCell(0).setCellValue("1.检查正常打“√”。");
            ExcelUtils.createVoidCell(1, 10, row31);
            sheet.addMergedRegion(new CellRangeAddress(nextRowNum, nextRowNum, 0, 10));

            nextRowNum++;
            XSSFRow row32 = sheet.createRow(nextRowNum);
            row32.createCell(0).setCellValue("2.检查有不正常情况打“X”，并记录存在问题，处理结果及完成时间。");
            ExcelUtils.createVoidCell(1, 10, row32);
            sheet.addMergedRegion(new CellRangeAddress(nextRowNum, nextRowNum, 0, 10));

            nextRowNum++;
            XSSFRow row33 = sheet.createRow(nextRowNum);
            row33.createCell(0).setCellValue("3. 检查频率为每一个月检查维护一次。");
            ExcelUtils.createVoidCell(1, 10, row33);
            sheet.addMergedRegion(new CellRangeAddress(nextRowNum, nextRowNum, 0, 10));

            //返回
            String fileName = "PSSR_13_消气防设施_气体保护系统检查记录.xlsx";
            OutputStream os = Files.newOutputStream(Paths.get(ExcelUtil.getAbsoluteFile(fileName)));
            workbook.write(os);
            workbook.close();
            return AjaxResult.success(fileName);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 获取CO2气体保护系统详细信息
     */
    @PreAuthorize("@ss.hasPermi('fire:shield:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id) {
        return AjaxResult.success(tFireGasShieldService.selectTFireGasShieldById(id));
    }

    /**
     * 获取CO2气体保护系统详细信息
     */
    @PreAuthorize("@ss.hasPermi('fire:shield:query')")
    @GetMapping(value = "/info")
    public AjaxResult getInfo( TFireGasShield tFireGasShield) {
        if (StringUtils.isEmpty(tFireGasShield.getDevNo()) || StringUtils.isNull(tFireGasShield.getPlanId())) {
            return AjaxResult.error("位号或计划id不能为空！");
        }
        TFireGasShield item = new TFireGasShield();
        item.setPlanId(tFireGasShield.getPlanId());
        item.setPrecision(tFireGasShield.getDevNo());
        item.setIssuesFlag("0");
        List<TFireGasShield> list = tFireGasShieldService.selectTFireGasShieldList(item);
        if (CollectionUtils.isNotEmpty(list))
            return AjaxResult.success(list.get(0));
        return AjaxResult.error("此设备不在当前巡检计划中!");
    }

    /**
     * 新增CO2气体保护系统
     */
    @PreAuthorize("@ss.hasPermi('fire:shield:add')")
    @Log(title = "CO2气体保护系统", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody TFireGasShield tFireGasShield) {
        return toAjax(tFireGasShieldService.insertTFireGasShield(tFireGasShield));
    }

    /**
     * 修改CO2气体保护系统
     */
    @PreAuthorize("@ss.hasPermi('fire:shield:edit')")
    @Log(title = "CO2气体保护系统", businessType = BusinessType.UPDATE)
    @PutMapping
    @RepeatSubmit
    public AjaxResult edit(@RequestBody TFireGasShield tFireGasShield) {
        tFireGasShield.setUpdatedate(new Date());
        tFireGasShield.setUpdaterCode(getUserId().toString());
        tFireGasShield.setCheckStatus("1");
        tFireGasShield.setCheckDate(new Date());
        TFireGasShield item = tFireGasShieldService.selectTFireGasShieldById(tFireGasShield.getId());
        TPatrolPlan tPatrolPlan = tPatrolPlanService.selectTPatrolPlanById(tFireGasShield.getPlanId());
        if (tPatrolPlan.getStatus()==1){
            return AjaxResult.error("当前选择的计划已确认！");
        }
        if (!"1".equals(item.getCheckStatus())) {
            tPatrolPlan.setUncheckedNum(tPatrolPlan.getUncheckedNum() - 1);
            tPatrolPlan.setCheckedNum(tPatrolPlan.getCheckedNum() + 1);
            tPatrolPlanService.updateTPatrolPlan(tPatrolPlan);
        }
        if ((StringUtils.isNotEmpty(tFireGasShield.getCardSign()) && !"1".equals(tFireGasShield.getCardSign())) ||
                (StringUtils.isNotEmpty(tFireGasShield.getComponentAppearance()) && !"1".equals(tFireGasShield.getComponentAppearance())) ||
                (StringUtils.isNotEmpty(tFireGasShield.getDeviceStatus()) && !"1".equals(tFireGasShield.getDeviceStatus())) ||
                (StringUtils.isNotEmpty(tFireGasShield.getIllumination()) && !"1".equals(tFireGasShield.getIllumination())) ||
                (StringUtils.isNotEmpty(tFireGasShield.getIndicatorStatus()) && !"1".equals(tFireGasShield.getIndicatorStatus())) ||
                (StringUtils.isNotEmpty(tFireGasShield.getSafetySign()) && !"1".equals(tFireGasShield.getSafetySign())) ||
                (StringUtils.isNotEmpty(tFireGasShield.getPressure()) && !"1".equals(tFireGasShield.getPressure())) ||
                (StringUtils.isNotEmpty(tFireGasShield.getReleaseSign()) && !"1".equals(tFireGasShield.getReleaseSign()))) {
            tFireGasShield.setIssuesStatus("1");
            tFireGasShield.setCheckStatus("0");
            tFireGasShieldService.updateTFireGasShield(tFireGasShield);
            tFireGasShield.setIssuesFlag("1");
            tFireGasShield.setCheckStatus("1");
            tFireGasShield.setCreatedate(new Date());
            tFireGasShield.setCreaterCode(getUserId().toString());
            tFireGasShield.setDevNo(item.getDevNo());
            tFireGasShieldService.insertTFireGasShield(tFireGasShield);
            TPatrolIssues tPatrolIssues = new TPatrolIssues();
            tPatrolIssues.setPatrolType("9");
            tPatrolIssues.setDevDescribe(tFireGasShield.getDevNo());
            tPatrolIssues.setLinkId(tFireGasShield.getId());
            tPatrolIssues.setStatus(0L);
            tPatrolIssuesService.insertTPatrolIssues(tPatrolIssues);
        } else {
            if ("1".equals(item.getIssuesStatus())) {
                TPatrolIssues tPatrolIssues = new TPatrolIssues();
                tPatrolIssues.setPatrolType("9");
                tPatrolIssues.setDevDescribe(item.getDevNo());
                tPatrolIssues.setCreatedate(new Date());
                for (TPatrolIssues patrolIssues : tPatrolIssuesService.selectTPatrolIssuesList(tPatrolIssues)) {
                    patrolIssues.setStatus(1L);
                    tPatrolIssuesService.updateTPatrolIssues(patrolIssues);
                }
            }
            tFireGasShield.setIssuesStatus("0");
            tFireGasShieldService.updateTFireGasShield(tFireGasShield);
        }
        return AjaxResult.success();
    }

    /**
     * 删除CO2气体保护系统
     */
    @PreAuthorize("@ss.hasPermi('fire:shield:remove')")
    @Log(title = "CO2气体保护系统", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids) {
        return toAjax(tFireGasShieldService.deleteTFireGasShieldByIds(ids));
    }
}
