package com.ruoyi.project.production.controller;

import java.util.Date;
import java.util.List;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.framework.aspectj.lang.annotation.Log;
import com.ruoyi.framework.aspectj.lang.enums.BusinessType;
import com.ruoyi.project.production.domain.TMonthlyQualityReportS2005;
import com.ruoyi.project.production.service.ITMonthlyQualityReportS2005Service;
import com.ruoyi.framework.web.controller.BaseController;
import com.ruoyi.framework.web.domain.AjaxResult;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.framework.web.page.TableDataInfo;

/**
 * 质量月报S2005Controller
 *
 * @author ruoyi
 * @date 2023-10-27
 */
@RestController
@RequestMapping("/production/s2005")
public class TMonthlyQualityReportS2005Controller extends BaseController
{
    @Autowired
    private ITMonthlyQualityReportS2005Service tMonthlyQualityReportS2005Service;

    /**
     * 查询质量月报S2005列表
     */
    @PreAuthorize("@ss.hasPermi('production:quality:list')")
    @GetMapping("/list")
    public AjaxResult list(TMonthlyQualityReportS2005 tMonthlyQualityReportS2005)
    {
        Date endDate = tMonthlyQualityReportS2005.getEndDate();
        endDate.setHours(23);
        endDate.setMinutes(59);
        endDate.setSeconds(59);
        tMonthlyQualityReportS2005.setEndDate(endDate);
        return AjaxResult.success(tMonthlyQualityReportS2005Service.selectTMonthlyQualityReportS2005List(tMonthlyQualityReportS2005));
    }

    /**
     * 导出质量月报S2005列表
     */
    @PreAuthorize("@ss.hasPermi('production:quality:export')")
    @Log(title = "质量月报S2005", businessType = BusinessType.EXPORT)
    @GetMapping("/export")
    public AjaxResult export(TMonthlyQualityReportS2005 tMonthlyQualityReportS2005)
    {
        List<TMonthlyQualityReportS2005> list = tMonthlyQualityReportS2005Service.selectTMonthlyQualityReportS2005List(tMonthlyQualityReportS2005);
        ExcelUtil<TMonthlyQualityReportS2005> util = new ExcelUtil<TMonthlyQualityReportS2005>(TMonthlyQualityReportS2005.class);
        return util.exportExcel(list, "quality");
    }

    /**
     * 获取质量月报S2005详细信息
     */
    @PreAuthorize("@ss.hasPermi('production:quality:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return AjaxResult.success(tMonthlyQualityReportS2005Service.selectTMonthlyQualityReportS2005ById(id));
    }

    /**
     * 新增质量月报S2005
     */
    @PreAuthorize("@ss.hasPermi('production:quality:add')")
    @Log(title = "质量月报S2005", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody TMonthlyQualityReportS2005 tMonthlyQualityReportS2005)
    {
        return toAjax(tMonthlyQualityReportS2005Service.insertTMonthlyQualityReportS2005(tMonthlyQualityReportS2005));
    }

    /**
     * 修改质量月报S2005
     */
    @PreAuthorize("@ss.hasPermi('production:quality:edit')")
    @Log(title = "质量月报S2005", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody TMonthlyQualityReportS2005 tMonthlyQualityReportS2005)
    {
        return toAjax(tMonthlyQualityReportS2005Service.updateTMonthlyQualityReportS2005(tMonthlyQualityReportS2005));
    }

    /**
     * 删除质量月报S2005
     */
    @PreAuthorize("@ss.hasPermi('production:quality:remove')")
    @Log(title = "质量月报S2005", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(tMonthlyQualityReportS2005Service.deleteTMonthlyQualityReportS2005ByIds(ids));
    }
}
