package com.ruoyi.project.production.controller;

import java.util.Date;
import java.util.List;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.framework.aspectj.lang.annotation.Log;
import com.ruoyi.framework.aspectj.lang.enums.BusinessType;
import com.ruoyi.project.production.domain.TMonthlyQualityReportS3012;
import com.ruoyi.project.production.service.ITMonthlyQualityReportS3012Service;
import com.ruoyi.framework.web.controller.BaseController;
import com.ruoyi.framework.web.domain.AjaxResult;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.framework.web.page.TableDataInfo;

/**
 * 质量月报S3012Controller
 *
 * @author ruoyi
 * @date 2023-10-27
 */
@RestController
@RequestMapping("/production/s3012")
public class TMonthlyQualityReportS3012Controller extends BaseController
{
    @Autowired
    private ITMonthlyQualityReportS3012Service tMonthlyQualityReportS3012Service;

    /**
     * 查询质量月报S3012列表
     */
    @PreAuthorize("@ss.hasPermi('production:quality:list')")
    @GetMapping("/list")
    public AjaxResult list(TMonthlyQualityReportS3012 tMonthlyQualityReportS3012)
    {
        Date endDate = tMonthlyQualityReportS3012.getEndDate();
        endDate.setHours(23);
        endDate.setMinutes(59);
        endDate.setSeconds(59);
        tMonthlyQualityReportS3012.setEndDate(endDate);
        return AjaxResult.success(tMonthlyQualityReportS3012Service.selectTMonthlyQualityReportS3012List(tMonthlyQualityReportS3012));
    }

    /**
     * 导出质量月报S3012列表
     */
    @PreAuthorize("@ss.hasPermi('production:quality:export')")
    @Log(title = "质量月报S3012", businessType = BusinessType.EXPORT)
    @GetMapping("/export")
    public AjaxResult export(TMonthlyQualityReportS3012 tMonthlyQualityReportS3012)
    {
        List<TMonthlyQualityReportS3012> list = tMonthlyQualityReportS3012Service.selectTMonthlyQualityReportS3012List(tMonthlyQualityReportS3012);
        ExcelUtil<TMonthlyQualityReportS3012> util = new ExcelUtil<TMonthlyQualityReportS3012>(TMonthlyQualityReportS3012.class);
        return util.exportExcel(list, "quality");
    }

    /**
     * 获取质量月报S3012详细信息
     */
    @PreAuthorize("@ss.hasPermi('production:quality:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return AjaxResult.success(tMonthlyQualityReportS3012Service.selectTMonthlyQualityReportS3012ById(id));
    }

    /**
     * 新增质量月报S3012
     */
    @PreAuthorize("@ss.hasPermi('production:quality:add')")
    @Log(title = "质量月报S3012", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody TMonthlyQualityReportS3012 tMonthlyQualityReportS3012)
    {
        return toAjax(tMonthlyQualityReportS3012Service.insertTMonthlyQualityReportS3012(tMonthlyQualityReportS3012));
    }

    /**
     * 修改质量月报S3012
     */
    @PreAuthorize("@ss.hasPermi('production:quality:edit')")
    @Log(title = "质量月报S3012", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody TMonthlyQualityReportS3012 tMonthlyQualityReportS3012)
    {
        return toAjax(tMonthlyQualityReportS3012Service.updateTMonthlyQualityReportS3012(tMonthlyQualityReportS3012));
    }

    /**
     * 删除质量月报S3012
     */
    @PreAuthorize("@ss.hasPermi('production:quality:remove')")
    @Log(title = "质量月报S3012", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(tMonthlyQualityReportS3012Service.deleteTMonthlyQualityReportS3012ByIds(ids));
    }
}
