package com.ruoyi.project.pssr.controller;

import com.alibaba.fastjson.JSON;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.file.ExcelUtils;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.framework.aspectj.lang.annotation.Log;
import com.ruoyi.framework.aspectj.lang.enums.BusinessType;
import com.ruoyi.framework.web.controller.BaseController;
import com.ruoyi.framework.web.domain.AjaxResult;
import com.ruoyi.framework.web.page.TableDataInfo;
import com.ruoyi.project.pssr.domain.TPssrApprove;
import com.ruoyi.project.pssr.domain.TPssrLaboratory;
import com.ruoyi.project.pssr.domain.TPssrSubcontent;
import com.ruoyi.project.pssr.domain.TPssrTurndown;
import com.ruoyi.project.pssr.mapper.TPssrLaboratoryMapper;
import com.ruoyi.project.pssr.service.*;
import com.ruoyi.project.system.domain.SysDept;
import com.ruoyi.project.system.domain.SysUser;
import com.ruoyi.project.system.service.ISysDeptService;
import com.ruoyi.project.system.service.ISysUserService;
import org.activiti.engine.ProcessEngine;
import org.activiti.engine.ProcessEngines;
import org.activiti.engine.TaskService;
import org.activiti.engine.task.Task;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * 实验室Controller
 *
 * @author ssy
 * @date 2024-10-30
 */
@RestController
@RequestMapping("/pssr/laboratory")
public class TPssrLaboratoryController extends BaseController {
    @Autowired
    private ITPssrFileService tPssrFileService;
    @Resource
    private TPssrLaboratoryMapper tPssrLaboratoryMapper;
    @Autowired
    private ITPssrTurndownService tPssrTurndownService;
    @Autowired
    private ITPssrLaboratoryService tPssrLaboratoryService;

    @Autowired
    private ITPssrApproveService tPssrApproveService;

    @Autowired
    private ITPssrSubcontentService tPssrSubcontentService;

    @Autowired
    private ISysUserService sysUserService;
    private String forShort = "sys";

    @Autowired
    private ISysDeptService iSysDeptService;

    @Autowired
    private ISysUserService userService;

    /**
     * 批量导入
     */
    @PreAuthorize("@ss.hasPermi('pssr:laboratory:add')")
    @PostMapping("/importData")
    public AjaxResult importInterlockData(MultipartFile file, Long subId) throws IOException
    {
        //获取操作人员ID
        Long userId = getUserId();
        //报错行数统计
        List<Integer> failRow =new ArrayList<Integer>();
        Workbook workbook = ExcelUtils.getWorkBook(file);
        Sheet sheet = workbook.getSheetAt(0);
        List<TPssrLaboratory> list = new ArrayList<TPssrLaboratory>();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        //部门查询
        List<SysDept> dept = iSysDeptService.selectDeptList(new SysDept());
        int rowNum = sheet.getPhysicalNumberOfRows();
        int failNumber = 0;
        for (int i = 2; i <= rowNum; i++) {
            try {
                logger.info("读取行数：" + i);
                Row row = sheet.getRow(i);
                int cellNum = row.getLastCellNum();
                TPssrLaboratory entity = new TPssrLaboratory();
                entity.setDeptId(userService.selectUserById(getUserId()).getDeptId());
                entity.setSubId(subId);
                entity.setApproveStatus(0L);
                for (int j = 0; j < cellNum; j++) {
                    Cell cell = row.getCell(j);
                    String cellValue = ExcelUtils.getCellValue(cell);
                    logger.info("cellValue:" + cellValue);
                    if (j == 0) {
                        entity.setInstrumentNumber(cellValue);
                    } else if (j == 1) {
                        entity.setName(cellValue);
                    } else if (j == 2) {
                        entity.setSerialNumber(cellValue);
                    }
                }
                entity.setCreaterCode(userId.toString());
                logger.info("entity:" + entity);
                list.add(entity);
            }catch (Exception e){
                failNumber++;
                failRow.add(i+1);
            }
        }
        int successNumber = 0;
        int failNum = 0;
        for (TPssrLaboratory t : list
        ) {
            failNum++;
            try {
                this.add(t);
                successNumber++;
            }catch (Exception e){
                failNumber++;
                logger.info("e:" + e);
                failRow.add(failNum+1);
            }
        }
        logger.info("list:" + JSON.toJSONString(list));
        logger.info("successNumber:" +String.valueOf(successNumber));
        logger.info("failNumber:" +String.valueOf(failNumber));
        logger.info("failRow:" +String.valueOf(failRow));
        return AjaxResult.success("导入成功行数：" + String.valueOf(successNumber));
    }

    /**
     * 查询实验室列表
     */
    @PreAuthorize("@ss.hasPermi('pssr:laboratory:list')")
    @GetMapping("/list")
    public TableDataInfo list(TPssrLaboratory tPssrLaboratory) {
        startPage();
        List<TPssrLaboratory> list = tPssrLaboratoryService.selectTPssrLaboratoryList(tPssrLaboratory);
        list.forEach(item -> {
            item.setFileList(tPssrFileService.selectTPssrFileListByItem(item.getSubId(), item.getId(), forShort));
            if (item.getApproveStatus() != 2)
                item.setReason(tPssrTurndownService.selectTPssrTurndownByItem(item.getSubId(), item.getId(), forShort));
        });
        return getDataTable(list);
    }

    /**
     * 导出实验室列表
     */
    @PreAuthorize("@ss.hasPermi('pssr:laboratory:export')")
    @Log(title = "实验室", businessType = BusinessType.EXPORT)
    @GetMapping("/export")
    public AjaxResult export(TPssrLaboratory tPssrLaboratory) {
        List<TPssrLaboratory> list = tPssrLaboratoryService.selectTPssrLaboratoryList(tPssrLaboratory);
        return AjaxResult.success(exportTmpl(list));
    }

    public String exportTmpl(List<TPssrLaboratory> list) {
        OutputStream out = null;
        String filename = null;
        try {
            String tempUrl = "static/word/pssr/sys.xlsx"; // 模板文件
            InputStream is = null;
            is = Thread.currentThread().getContextClassLoader().getResourceAsStream(tempUrl);
            XSSFWorkbook wb = null;
            wb = new XSSFWorkbook(is);
            XSSFSheet sheet = wb.getSheetAt(0);

            //填充数据
            int rowIndex = 3;
            int num = 1;

            Row originalRow = sheet.getRow(3);
            Cell originalcell = originalRow.getCell(0);
            // 获取单元格样式
            CellStyle originalStyle = originalcell.getCellStyle();

            for (TPssrLaboratory t : list) {
                Row row = sheet.createRow(rowIndex);
                row.setHeight((short) 800);
                row.createCell(0).setCellValue(num);
                row.createCell(1).setCellValue(t.getInstrumentNumber());
                row.createCell(2).setCellValue(t.getName());
                row.createCell(3).setCellValue(t.getSerialNumber());
                row.createCell(4).setCellValue(t.getVisualInspection());
                row.createCell(5).setCellValue(t.getFunctionalCheck());
                row.createCell(6);
                row.createCell(7);
                try {
                    SysUser sysUser = sysUserService.selectUserById(Long.valueOf(t.getConfirm1()));
                    SysUser sysUser2 = sysUserService.selectUserById(Long.valueOf(t.getConfirm2()));
                    String confirm1 = sysUser.getSignUrl();
                    String confirm2 = sysUser2.getSignUrl();
                    ExcelUtils.insertPicture(wb, sheet, confirm1, row.getRowNum(), 6, 1, 1);
                    ExcelUtils.insertPicture(wb, sheet, confirm2, row.getRowNum(), 7, 1, 1);
                } catch (NumberFormatException e) {
                    throw new RuntimeException(e);
                }
                row.createCell(8).setCellValue(DateUtils.dateTime(t.getConfirmationDate()));
                row.createCell(9).setCellValue(t.getRemarks());


                //渲染样式
                for (int i = 0; i < 10; i++) {
                    row.getCell(i).setCellStyle(originalStyle);
                }
                num++;
                rowIndex++;
            }
            filename = "PSSR_10_实验室"  + ".xlsx";
            out = new FileOutputStream(ExcelUtil.getAbsoluteFile(filename));
            wb.write(out);
            wb.close();

        } catch (IOException e) {
            e.printStackTrace();
        }
        return filename;
    }

    /**
     * 获取实验室详细信息
     */
    @PreAuthorize("@ss.hasPermi('pssr:laboratory:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        TPssrLaboratory item = tPssrLaboratoryService.selectTPssrLaboratoryById(id);
        item.setFileList(tPssrFileService.selectTPssrFileListByItem(item.getSubId(), item.getId(), forShort));
        if (item.getApproveStatus() != 2)
            item.setReason(tPssrTurndownService.selectTPssrTurndownByItem(item.getSubId(), item.getId(), forShort));
        return AjaxResult.success(item);
    }

    /**
     * 新增实验室
     */
    @PreAuthorize("@ss.hasPermi('pssr:laboratory:add')")
    @Log(title = "实验室", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody TPssrLaboratory tPssrLaboratory) {
        if (StringUtils.isNotEmpty(tPssrLaboratory.getConfirm1())&&tPssrLaboratory.getConfirm1().equals(tPssrLaboratory.getConfirm2())) {
            return AjaxResult.error("确认人不能为同一人，请重新选择！");
        }
        tPssrLaboratory.setApproveStatus(0L);
        return toAjax(tPssrLaboratoryService.insertTPssrLaboratory(tPssrLaboratory));
    }

    /**
     * 修改实验室
     */
    @PreAuthorize("@ss.hasPermi('pssr:laboratory:edit')")
    @Log(title = "实验室", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody TPssrLaboratory tPssrLaboratory)
    {
        if (tPssrLaboratory.getConfirm1().equals(tPssrLaboratory.getConfirm2())){
            return AjaxResult.error("确认人不能为同一人，请重新选择！");
        }
        tPssrFileService.updateFileRelevance(tPssrLaboratory.getFileIds(), forShort, tPssrLaboratory.getId(), tPssrLaboratory.getSubId());
        return toAjax(tPssrLaboratoryService.updateTPssrLaboratory(tPssrLaboratory));
    }

    /**
     * 修改实验室
     */
    @PreAuthorize("@ss.hasPermi('pssr:laboratory:edit')")
    @Log(title = "实验室", businessType = BusinessType.UPDATE)
    @PutMapping("/editBatch")
    public AjaxResult editb(@RequestBody TPssrLaboratory tPssrLaboratory)
    {
        if (tPssrLaboratory.getConfirm1().equals(tPssrLaboratory.getConfirm2())){
            return AjaxResult.error("确认人不能为同一人，请重新选择！");
        }
        return toAjax(tPssrLaboratoryMapper.updateTPssrLaboratoryByIds(tPssrLaboratory));
    }

    /**
     * 删除实验室
     */
    @PreAuthorize("@ss.hasPermi('pssr:laboratory:remove')")
    @Log(title = "实验室", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids) {
        return toAjax(tPssrLaboratoryService.deleteTPssrLaboratoryByIds(ids));
    }

    /**
     * 确认实验室
     */
    @PreAuthorize("@ss.hasPermi('pssr:laboratory:edit')")
    @Log(title = "实验室", businessType = BusinessType.UPDATE)
    @PutMapping("/confirmLaboratory")
    public AjaxResult confirmLaboratory(@RequestBody TPssrLaboratory tPssrLaboratory) {
        long queryStatus = 0;
        long approveStatus = 0;
        Date date = null;
        if (tPssrLaboratory.getTaskType() == 4) {
            //拆锁确认
            queryStatus = 1;
            approveStatus = 3;
            date = new Date();
        } else if (tPssrLaboratory.getTaskType() == 5) {
            //上锁确认
            queryStatus = 3;
            approveStatus = 2;
            date = new Date();
        }
        // 修改状态
        if (tPssrLaboratory.getIds() != null && tPssrLaboratory.getIds().length > 0) {
            for (Long id : tPssrLaboratory.getIds()) {
                TPssrLaboratory item = tPssrLaboratoryService.selectTPssrLaboratoryById(id);
                item.setApproveStatus(approveStatus);
                if (queryStatus == 3) {
                    item.setConfirmationDate(date);
                }
                tPssrLaboratoryService.updateTPssrLaboratory(item);
            }
        } else {
            TPssrLaboratory lock = new TPssrLaboratory();
            lock.setSubId(tPssrLaboratory.getSubId());
            lock.setApproveStatus(queryStatus);
            for (TPssrLaboratory item : tPssrLaboratoryService.selectTPssrLaboratoryList(lock)) {
                if (queryStatus == 3) {
                    item.setConfirmationDate(date);
                }
                item.setApproveStatus(approveStatus);
                tPssrLaboratoryService.updateTPssrLaboratory(item);
            }
        }
        //查询当前待审批的确认人
        TPssrLaboratory entity = new TPssrLaboratory();
        entity.setSubId(tPssrLaboratory.getSubId());
        entity.setApproveStatus(queryStatus);
        for (TPssrLaboratory item : tPssrLaboratoryService.selectTPssrLaboratoryList(entity)) {
            if (tPssrLaboratory.getTaskType() == 4) {
                if (item.getConfirm1().equals(getUserId().toString())) {
                    return AjaxResult.success();
                }
            } else if (tPssrLaboratory.getTaskType() == 5) {
                if (item.getConfirm2().equals(getUserId().toString())) {
                    return AjaxResult.success();
                }
            }
        }

        //无待审批任务结束当前用户流程

        // 因为流程关系所以approve一定会有且只有一条数据
        TPssrApprove tPssrApprove = tPssrApproveService.selectTPssrApproveBySubId(tPssrLaboratory.getSubId());
        TPssrApproveController.handleConfirmApprove(tPssrApprove, getUserId().toString());
        return AjaxResult.success();

    }

    /**
     * 驳回实验室
     */
    @PutMapping("/turnDownLaboratory")
    public AjaxResult turnDownLaboratory(@RequestBody List<TPssrLaboratory> tPssrLaboratory) {
        if (CollectionUtils.isNotEmpty(tPssrLaboratory)) {
            String userId = getUserId().toString();
            Long subId = tPssrLaboratory.get(0).getSubId();
            // 修改已选择数据的状态
            for (TPssrLaboratory item : tPssrLaboratory) {
                TPssrLaboratory blind = new TPssrLaboratory();
                blind.setId(item.getId());
                blind.setApproveStatus(1L);
                blind.setUpdatedate(new Date());
                blind.setUpdaterCode(getUserId().toString());
                tPssrLaboratoryService.updateTPssrLaboratory(blind);
                // 新增驳回原因数据
                TPssrTurndown turndown = new TPssrTurndown();
                turndown.setForShort(forShort);
                turndown.setSubId(item.getSubId());
                turndown.setItemId(item.getId());
                turndown.setReason(item.getReason());
                turndown.setCreatedate(new Date());
                turndown.setCreaterCode(getUserId().toString());
                tPssrTurndownService.insertTPssrTurndown(turndown);
            }
            // 查询当前流程
            TPssrApprove approve = tPssrApproveService.selectTPssrApproveBySubId(subId);

            ProcessEngine processEngine = ProcessEngines.getDefaultProcessEngine();
            TaskService taskService = processEngine.getTaskService();
            Task task = processEngine.getTaskService()//获取任务service
                    .createTaskQuery()//创建查询对象
                    .taskAssignee(userId)
                    .processInstanceId(approve.getProcessId()).singleResult();
            String taskId = task.getId();

            // 驳回 查询所有待审批的人员
            TPssrLaboratory blind = new TPssrLaboratory();
            blind.setSubId(subId);
            blind.setApproveStatus(1L);
            Set<String> installer = new HashSet<>();
            Set<String> remover = new HashSet<>();
            for (TPssrLaboratory item : tPssrLaboratoryService.selectTPssrLaboratoryList(blind)) {

                // 安装人员
                installer.add(item.getConfirm1());

                //拆除人员
                remover.add(item.getConfirm2());
            }


            //处理流程节点
            Map<String, Object> param = new HashMap<>();
            param.put("condition", 1);
            param.put("confirmUsers1", new ArrayList<>(installer));
            param.put("confirmUsers2", new ArrayList<>(remover));
            //认领任务
            processEngine.getTaskService().claim(taskId, userId);
            taskService.addComment(taskId, approve.getProcessId(), "驳回");
            taskService.complete(taskId, param);

            // 修改审批表和sub表
            approve.setApproveStatus(1L);
            approve.setUpdatedate(new Date());
            approve.setUpdaterCode(getUserId().toString());
            tPssrApproveService.updateTPssrApprove(approve);

            TPssrSubcontent subcontent = new TPssrSubcontent();
            subcontent.setId(approve.getSubId());
            subcontent.setApproveStatus(1L);
            subcontent.setUpdatedate(new Date());
            subcontent.setUpdaterCode(getUserId().toString());
            tPssrSubcontentService.updateTPssrSubcontent(subcontent);
            return AjaxResult.success();
        }
        return AjaxResult.error();
    }
}
