package com.ruoyi.project.pssr.controller;

import com.alibaba.fastjson.JSON;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.file.ExcelUtils;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.framework.aspectj.lang.annotation.Log;
import com.ruoyi.framework.aspectj.lang.enums.BusinessType;
import com.ruoyi.framework.web.controller.BaseController;
import com.ruoyi.framework.web.domain.AjaxResult;
import com.ruoyi.framework.web.page.TableDataInfo;
import com.ruoyi.project.listener.pssr.ConfirmTaskCreateListener;
import com.ruoyi.project.pssr.domain.TPssrApprove;
import com.ruoyi.project.pssr.domain.TPssrProtection;
import com.ruoyi.project.pssr.domain.TPssrSubcontent;
import com.ruoyi.project.pssr.domain.TPssrTurndown;
import com.ruoyi.project.pssr.mapper.TPssrProtectionMapper;
import com.ruoyi.project.pssr.service.*;
import com.ruoyi.project.system.domain.SysDept;
import com.ruoyi.project.system.domain.SysUser;
import com.ruoyi.project.system.service.ISysDeptService;
import com.ruoyi.project.system.service.ISysUserService;
import org.activiti.engine.HistoryService;
import org.activiti.engine.RuntimeService;
import org.activiti.engine.impl.identity.Authentication;
import org.activiti.engine.runtime.ProcessInstance;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * 人身防护Controller
 *
 * @author ssy
 * @date 2024-09-18
 */
@RestController
@RequestMapping("/pssr/protection")
public class TPssrProtectionController extends BaseController {
    @Resource
     private TPssrProtectionMapper tPssrProtectionMapper;
    @Autowired
    private ITPssrFileService tPssrFileService;

    @Autowired
    private ITPssrTurndownService tPssrTurndownService;
    @Autowired
    private ITPssrProtectionService tPssrProtectionService;

    @Autowired
    private ITPssrApproveService tPssrApproveService;

    @Autowired
    private ITPssrSubcontentService tPssrSubcontentService;

    @Autowired
    private RuntimeService runtimeService;

    @Autowired
    private HistoryService historyService;

    @Autowired
    private ISysUserService sysUserService;
    private String forShort = "rsfh";

    @Autowired
    private ISysDeptService iSysDeptService;

    @Autowired
    private ISysUserService userService;

    /**
     * 批量导入
     */
    @PreAuthorize("@ss.hasPermi('pssr:protection:add')")
    @PostMapping("/importData")
    public AjaxResult importInterlockData(MultipartFile file, Long subId) throws IOException
    {
        //获取操作人员ID
        Long userId = getUserId();
        //报错行数统计
        List<Integer> failRow =new ArrayList<Integer>();
        Workbook workbook = ExcelUtils.getWorkBook(file);
        Sheet sheet = workbook.getSheetAt(0);
        List<TPssrProtection> list = new ArrayList<TPssrProtection>();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        //部门查询
        List<SysDept> dept = iSysDeptService.selectDeptList(new SysDept());
        int rowNum = sheet.getPhysicalNumberOfRows();
        int failNumber = 0;
        for (int i = 2; i <= rowNum; i++) {
            try {
                logger.info("读取行数：" + i);
                Row row = sheet.getRow(i);
                int cellNum = row.getLastCellNum();
                TPssrProtection entity = new TPssrProtection();
                entity.setDeptId(userService.selectUserById(getUserId()).getDeptId());
                entity.setSubId(subId);
                entity.setApproveStatus(0L);
                for (int j = 0; j < cellNum; j++) {
                    Cell cell = row.getCell(j);
                    String cellValue = ExcelUtils.getCellValue(cell);
                    logger.info("cellValue:" + cellValue);
                    if (j == 0) {
                        entity.setCheckContent(cellValue);
                    } else if (j == 1) {
                        entity.setCheckResult(cellValue);
                    } else if (j == 2) {
                        if (cellValue.length() > 3) {
                            entity.setConfirmationDate(new SimpleDateFormat(DateUtils.getDateFormat(cellValue)).parse(cellValue));
                        }
                    } else if (j == 3) {
                        entity.setRemarks(cellValue);
                    }
                }
                entity.setCreaterCode(userId.toString());
                logger.info("entity:" + entity);
                list.add(entity);
            }catch (Exception e){
                failNumber++;
                failRow.add(i+1);
            }
        }
        int successNumber = 0;
        int failNum = 0;
        for (TPssrProtection t : list
        ) {
            failNum++;
            try {
                this.add(t);
                successNumber++;
            }catch (Exception e){
                failNumber++;
                logger.info("e:" + e);
                failRow.add(failNum+1);
            }
        }
        logger.info("list:" + JSON.toJSONString(list));
        logger.info("successNumber:" +String.valueOf(successNumber));
        logger.info("failNumber:" +String.valueOf(failNumber));
        logger.info("failRow:" +String.valueOf(failRow));
        return AjaxResult.success("导入成功行数：" + String.valueOf(successNumber));
    }

    /**
     * 查询人身防护列表
     */
    @PreAuthorize("@ss.hasPermi('pssr:protection:list')")
    @GetMapping("/list")
    public TableDataInfo list(TPssrProtection tPssrProtection) {
        startPage();
        List<TPssrProtection> list = tPssrProtectionService.selectTPssrProtectionList(tPssrProtection);
        list.forEach(item -> {
            item.setFileList(tPssrFileService.selectTPssrFileListByItem(item.getSubId(), item.getId(), "rsfh"));
            if (item.getApproveStatus() != 2)
                item.setReason(tPssrTurndownService.selectTPssrTurndownByItem(item.getSubId(), item.getId(), "rsfh"));
        });
        return getDataTable(list);
    }

    /**
     * 导出人身防护列表
     */
    @PreAuthorize("@ss.hasPermi('pssr:protection:export')")
    @Log(title = "人身防护", businessType = BusinessType.EXPORT)
    @GetMapping("/export")
    public AjaxResult export(TPssrProtection tPssrProtection) {
        List<TPssrProtection> list = tPssrProtectionService.selectTPssrProtectionList(tPssrProtection);
        return AjaxResult.success(exportTmpl(list));
    }

    public String exportTmpl(List<TPssrProtection> list) {
        OutputStream out = null;
        String filename = null;
        try {
            String tempUrl = "static/word/pssr/rsfh1.xlsx"; // 模板文件
            InputStream is = null;
            is = Thread.currentThread().getContextClassLoader().getResourceAsStream(tempUrl);
            XSSFWorkbook wb = null;
            wb = new XSSFWorkbook(is);
            XSSFSheet sheet = wb.getSheetAt(0);

            //填充数据
            int rowIndex = 2;
            int num = 1;

            Row originalRow = sheet.getRow(2);
            Cell originalcell = originalRow.getCell(0);
            // 获取单元格样式
            CellStyle originalStyle = originalcell.getCellStyle();

            for (TPssrProtection t : list) {
                Row row = sheet.createRow(rowIndex);
                row.setHeight((short) 800);
                row.createCell(0).setCellValue(num);
                row.createCell(1).setCellValue(t.getCheckContent());
                row.createCell(2).setCellValue(t.getCheckResult());
                row.createCell(3);
                row.createCell(4);
                try {
                    SysUser sysUser1 = sysUserService.selectUserById(Long.valueOf(t.getConfirm1()));
                    SysUser sysUser2 = sysUserService.selectUserById(Long.valueOf(t.getConfirm2()));
                    String confirm1 = sysUser1.getSignUrl();
                    String confirm2 = sysUser2.getSignUrl();
                    ExcelUtils.insertPicture(wb, sheet, confirm1, row.getRowNum(), 3, 1, 1);
                    ExcelUtils.insertPicture(wb, sheet, confirm2, row.getRowNum(), 4, 1, 1);
                } catch (NumberFormatException e) {
                    throw new RuntimeException(e);
                }
                row.createCell(5).setCellValue(DateUtils.dateTime(t.getConfirmationDate()));
                row.createCell(6).setCellValue(t.getRemarks());

                //渲染样式
                for (int i = 0; i < 7; i++) {
                    row.getCell(i).setCellStyle(originalStyle);
                }
                num++;
                rowIndex++;
            }
            filename = "PSSR_06_人身防护_现场管道和设备保温防护检查表"  + ".xlsx";
            out = new FileOutputStream(ExcelUtil.getAbsoluteFile(filename));
            wb.write(out);
            wb.close();

        } catch (IOException e) {
            e.printStackTrace();
        }
        return filename;
    }

    /**
     * 获取人身防护详细信息
     */
    @PreAuthorize("@ss.hasPermi('pssr:protection:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id) {
        TPssrProtection item = tPssrProtectionService.selectTPssrProtectionById(id);
        item.setFileList(tPssrFileService.selectTPssrFileListByItem(item.getSubId(), item.getId(), "rsfh"));
            if (item.getApproveStatus() != 2)
                item.setReason(tPssrTurndownService.selectTPssrTurndownByItem(item.getSubId(), item.getId(), "rsfh"));
        return AjaxResult.success(item);
    }

    /**
     * 新增人身防护
     */
    @PreAuthorize("@ss.hasPermi('pssr:protection:add')")
    @Log(title = "人身防护", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody TPssrProtection tPssrProtection) {
        if (StringUtils.isNotEmpty(tPssrProtection.getConfirm1())&&tPssrProtection.getConfirm1().equals(tPssrProtection.getConfirm2())) {
            return AjaxResult.error("确认人不能为同一人，请重新选择！");
        }
        tPssrProtection.setApproveStatus(0L);
        tPssrProtection.setCreaterCode(String.valueOf(getUserId()));
        tPssrProtection.setCreatedate(new Date());
        return toAjax(tPssrProtectionService.insertTPssrProtection(tPssrProtection));
    }

    /**
     * 修改人身防护
     */
    @PreAuthorize("@ss.hasPermi('pssr:protection:edit')")
    @Log(title = "人身防护", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody TPssrProtection tPssrProtection) {
        if (tPssrProtection.getConfirm1().equals(tPssrProtection.getConfirm2())){
            return AjaxResult.error("确认人不能为同一人，请重新选择！");
        }
        tPssrFileService.updateFileRelevance(tPssrProtection.getFileIds(), "rsfh", tPssrProtection.getId(), tPssrProtection.getSubId());
        return toAjax(tPssrProtectionService.updateTPssrProtection(tPssrProtection));
    }

    /**
     * 修改人身防护
     */
    @PreAuthorize("@ss.hasPermi('pssr:protection:edit')")
    @Log(title = "人身防护", businessType = BusinessType.UPDATE)
    @PutMapping("/editBatch")
    public AjaxResult editb(@RequestBody TPssrProtection tPssrProtection) {
        if (tPssrProtection.getConfirm1().equals(tPssrProtection.getConfirm2())){
            return AjaxResult.error("确认人不能为同一人，请重新选择！");
        }
        return toAjax(tPssrProtectionMapper.updateTPssrProtectionByIds(tPssrProtection));
    }

    /**
     * 删除人身防护
     */
    @PreAuthorize("@ss.hasPermi('pssr:protection:remove')")
    @Log(title = "人身防护", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids) {
        return toAjax(tPssrProtectionService.deleteTPssrProtectionByIds(ids));
    }

    /**
     * 确认人身防护
     */
    @PreAuthorize("@ss.hasPermi('pssr:protection:edit')")
    @Log(title = "人身防护", businessType = BusinessType.UPDATE)
    @PutMapping("/confirmProtection")
    public AjaxResult confirmProtection(@RequestBody TPssrProtection tPssrProtection) {
        long queryStatus = 0;
        long approveStatus = 0;
        Date date = null;
        TPssrProtection protection = new TPssrProtection();
        if (tPssrProtection.getTaskType() == 4) {
            //确认人1确认
            queryStatus = 1;
            approveStatus = 3;
            protection.setConfirm1(getUserId().toString());
            date = new Date();
        } else if (tPssrProtection.getTaskType() == 5) {
            //确认人2确认
            queryStatus = 3;
            approveStatus = 2;
            protection.setConfirm2(getUserId().toString());
            date = new Date();
        }
        if (tPssrProtection.getIds() != null && tPssrProtection.getIds().length > 0) {
            for (Long id : tPssrProtection.getIds()) {
                protection = tPssrProtectionService.selectTPssrProtectionById(id);
                if (queryStatus == 3) {
                    protection.setConfirmationDate(new Date());
                }
                protection.setApproveStatus(approveStatus);
                protection.setUpdatedate(new Date());
                protection.setUpdaterCode(String.valueOf(getUserId()));
                tPssrProtectionService.updateTPssrProtection(protection);
            }
        } else {
            protection.setSubId(tPssrProtection.getSubId());
            protection.setApproveStatus(queryStatus);
            for (TPssrProtection item : tPssrProtectionService.selectTPssrProtectionList(protection)) {
                if (queryStatus == 3) {
                    item.setConfirmationDate(new Date());
                }
                item.setApproveStatus(approveStatus);
                item.setUpdatedate(new Date());
                item.setUpdaterCode(String.valueOf(getUserId()));
                tPssrProtectionService.updateTPssrProtection(item);
            }
        }
        //查询当前待审批的确认人
        TPssrProtection entity = new TPssrProtection();
        entity.setSubId(tPssrProtection.getSubId());
        entity.setApproveStatus(queryStatus);
        TPssrProtection pssrProtection = tPssrProtectionService.selectAllConfirmedPersonBySubId(entity);
        if (pssrProtection != null) {
            //如果当前用户还有待审批任务
            if (tPssrProtection.getTaskType() == 4 && StringUtils.isNotEmpty(pssrProtection.getConfirm1())) {
                if (pssrProtection.getConfirm1().contains(getUserId().toString())) {
                    return AjaxResult.success();
                }
            }

            if (tPssrProtection.getTaskType() == 5 && StringUtils.isNotEmpty(pssrProtection.getConfirm2())) {
                if (pssrProtection.getConfirm2().contains(getUserId().toString())) {
                    return AjaxResult.success();
                }
            }
        }
        //无待审批任务结束当前用户流程

        // 因为流程关系所以approve一定会有且只有一条数据
        TPssrApprove tPssrApprove = tPssrApproveService.selectTPssrApproveBySubId(tPssrProtection.getSubId());
        TPssrApproveController.handleConfirmApprove(tPssrApprove, getUserId().toString());
        return AjaxResult.success();
    }


    /**
     * 驳回人身防护
     */
    @PutMapping("/turnDownProtection")
    public AjaxResult turnDownProtection(@RequestBody List<TPssrProtection> tPssrProtection) {
        if (CollectionUtils.isNotEmpty(tPssrProtection)) {
            String userId = getUserId().toString();
            Long subId = tPssrProtection.get(0).getSubId();
            // 修改已选择数据的状态
            for (TPssrProtection item : tPssrProtection) {
                TPssrProtection blind = new TPssrProtection();
                blind.setId(item.getId());
                blind.setApproveStatus(1L);
                blind.setUpdatedate(new Date());
                blind.setUpdaterCode(getUserId().toString());
                tPssrProtectionService.updateTPssrProtection(blind);
                // 新增驳回原因数据
                TPssrTurndown turndown = new TPssrTurndown();
                turndown.setForShort(forShort);
                turndown.setSubId(item.getSubId());
                turndown.setItemId(item.getId());
                turndown.setReason(item.getReason());
                turndown.setCreatedate(new Date());
                turndown.setCreaterCode(getUserId().toString());
                tPssrTurndownService.insertTPssrTurndown(turndown);
            }
            // 查询当前流程
            TPssrApprove approve = tPssrApproveService.selectTPssrApproveBySubId(subId);

            try {
                runtimeService.deleteProcessInstance(approve.getProcessId(), "pssr2confirm");
                historyService.deleteHistoricProcessInstance(approve.getProcessId());
            } catch (Exception e) {
                logger.info("无运行时流程");
            }

            // 驳回 查询所有待审批的人员
            TPssrProtection blind = new TPssrProtection();
            blind.setSubId(subId);
            blind.setApproveStatus(1L);
            Set<String> confirmUsers1 = new HashSet<>();
            Set<String> confirmUsers2 = new HashSet<>();
            for (TPssrProtection item : tPssrProtectionService.selectTPssrProtectionList(blind)) {

                // 确认人1
                confirmUsers1.add(item.getConfirm1());

                //确认人2
                confirmUsers2.add(item.getConfirm2());
            }

            // 开始申请流程
            long businessKey = approve.getApproveId();
            //开始工作流、监听
            Authentication.setAuthenticatedUserId(userId);//设置当前申请人
            Map<String, Object> variables = new HashMap<>();
            variables.put("applyUser", userId);
            variables.put("confirmUsers1", confirmUsers1);
            variables.put("confirmUsers2", confirmUsers2);
            variables.put("confirmTaskCreateListener", new ConfirmTaskCreateListener());//发送邮件
            variables.put("chargePerson", approve.getSubCharge());
            //采用key来启动流程定义并设置流程变量，返回流程实例
            ProcessInstance pi = runtimeService.startProcessInstanceByKey("pssr2confirm", String.valueOf(businessKey), variables);

            // 修改审批表和sub表
            approve.setProcessId(pi.getProcessInstanceId());
            approve.setApproveStatus(1L);
            approve.setUpdatedate(new Date());
            approve.setUpdaterCode(getUserId().toString());
            tPssrApproveService.updateTPssrApprove(approve);

            TPssrSubcontent subcontent = new TPssrSubcontent();
            subcontent.setId(approve.getSubId());
            subcontent.setApproveStatus(1L);
            subcontent.setUpdatedate(new Date());
            subcontent.setUpdaterCode(getUserId().toString());
            tPssrSubcontentService.updateTPssrSubcontent(subcontent);
            return AjaxResult.success();
        }
        return AjaxResult.error();
    }
}
