package com.ruoyi.project.system.service.impl;

import java.util.List;

import com.ruoyi.project.system.domain.SysUser;
import com.ruoyi.project.system.service.ISysUserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.project.system.mapper.TAlarmPrincipalMapper;
import com.ruoyi.project.system.domain.TAlarmPrincipal;
import com.ruoyi.project.system.service.ITAlarmPrincipalService;

import javax.annotation.Resource;

/**
 * 预警管理负责人Service业务层处理
 *
 * @author ruoyi
 * @date 2020-12-30
 */
@Service
public class TAlarmPrincipalServiceImpl implements ITAlarmPrincipalService
{
    @Resource
    private TAlarmPrincipalMapper tAlarmPrincipalMapper;

    @Autowired
    ISysUserService sysUserService;

    /**
     * 查询预警管理负责人
     *
     * @param id 预警管理负责人ID
     * @return 预警管理负责人
     */
    @Override
    public TAlarmPrincipal selectTAlarmPrincipalById(Long id)
    {
        return tAlarmPrincipalMapper.selectTAlarmPrincipalById(id);
    }

    /**
     * 查询预警管理负责人列表
     *
     * @param tAlarmPrincipal 预警管理负责人
     * @return 预警管理负责人
     */
    @Override
    public List<TAlarmPrincipal> selectTAlarmPrincipalList(TAlarmPrincipal tAlarmPrincipal)
    {
        return tAlarmPrincipalMapper.selectTAlarmPrincipalList(tAlarmPrincipal);
    }

    /**
     * 定时任务查询预警管理负责人列表
     *
     * @param tAlarmPrincipal 预警管理负责人
     * @return 预警管理负责人
     */
    @Override
    public List<TAlarmPrincipal> selectList(TAlarmPrincipal tAlarmPrincipal)
    {
        List<TAlarmPrincipal> list = tAlarmPrincipalMapper.selectList(tAlarmPrincipal);
        for (TAlarmPrincipal t: list
        ) {
            SysUser user = sysUserService.selectUserByStaffId(t.getStaffid());
            if (user != null) {
                t.setPrincipalName(user.getNickName());
                t.setPrincipalEmail(user.getEmail());
            }
        }
        return list;
    }

    /**
     * 新增预警管理负责人
     *
     * @param tAlarmPrincipal 预警管理负责人
     * @return 结果
     */
    @Override
    public int insertTAlarmPrincipal(TAlarmPrincipal tAlarmPrincipal)
    {
        return tAlarmPrincipalMapper.insertTAlarmPrincipal(tAlarmPrincipal);
    }

    /**
     * 修改预警管理负责人
     *
     * @param tAlarmPrincipal 预警管理负责人
     * @return 结果
     */
    @Override
    public int updateTAlarmPrincipal(TAlarmPrincipal tAlarmPrincipal)
    {
        return tAlarmPrincipalMapper.updateTAlarmPrincipal(tAlarmPrincipal);
    }

    /**
     * 批量删除预警管理负责人
     *
     * @param ids 需要删除的预警管理负责人ID
     * @return 结果
     */
    @Override
    public int deleteTAlarmPrincipalByIds(Long[] ids)
    {
        return tAlarmPrincipalMapper.deleteTAlarmPrincipalByIds(ids);
    }

    /**
     * 删除预警管理负责人信息
     *
     * @param id 预警管理负责人ID
     * @return 结果
     */
    @Override
    public int deleteTAlarmPrincipalById(Long id)
    {
        return tAlarmPrincipalMapper.deleteTAlarmPrincipalById(id);
    }
}
