package com.ruoyi.project.training.bccnew.controller;

import java.util.List;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.framework.aspectj.lang.annotation.Log;
import com.ruoyi.framework.aspectj.lang.enums.BusinessType;
import com.ruoyi.project.training.bccnew.domain.TTsFlplanContent;
import com.ruoyi.project.training.bccnew.service.ITTsFlplanContentService;
import com.ruoyi.framework.web.controller.BaseController;
import com.ruoyi.framework.web.domain.AjaxResult;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.framework.web.page.TableDataInfo;

/**
 * 分离培训内容Controller
 *
 * @author ssy
 * @date 2024-07-19
 */
@RestController
@RequestMapping("/bccnew/flContent")
public class TTsFlplanContentController extends BaseController
{
    @Autowired
    private ITTsFlplanContentService tTsFlplanContentService;

    /**
     * 查询分离培训内容列表
     */
    //@PreAuthorize("@ss.hasPermi('bccnew:flContent:list')")
    @GetMapping("/list")
    public AjaxResult list(TTsFlplanContent tTsFlplanContent)
    {
        return AjaxResult.success(tTsFlplanContentService.selectTTsFlplanContentList(tTsFlplanContent));
    }

    /**
     * 导出分离培训内容列表
     */
    //@PreAuthorize("@ss.hasPermi('bccnew:flContent:export')")
    @Log(title = "分离培训内容", businessType = BusinessType.EXPORT)
    @GetMapping("/export")
    public AjaxResult export(TTsFlplanContent tTsFlplanContent)
    {
        List<TTsFlplanContent> list = tTsFlplanContentService.selectTTsFlplanContentList(tTsFlplanContent);
        ExcelUtil<TTsFlplanContent> util = new ExcelUtil<TTsFlplanContent>(TTsFlplanContent.class);
        return util.exportExcel(list, "flContent");
    }

    /**
     * 获取分离培训内容详细信息
     */
    //@PreAuthorize("@ss.hasPermi('bccnew:flContent:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return AjaxResult.success(tTsFlplanContentService.selectTTsFlplanContentById(id));
    }

    /**
     * 新增分离培训内容
     */
    //@PreAuthorize("@ss.hasPermi('bccnew:flContent:add')")
    @Log(title = "分离培训内容", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody TTsFlplanContent tTsFlplanContent)
    {
        return toAjax(tTsFlplanContentService.insertTTsFlplanContent(tTsFlplanContent));
    }

    /**
     * 修改分离培训内容
     */
    //@PreAuthorize("@ss.hasPermi('bccnew:flContent:edit')")
    @Log(title = "分离培训内容", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody TTsFlplanContent tTsFlplanContent)
    {
        return toAjax(tTsFlplanContentService.updateTTsFlplanContent(tTsFlplanContent));
    }

    /**
     * 删除分离培训内容
     */
    //@PreAuthorize("@ss.hasPermi('bccnew:flContent:remove')")
    @Log(title = "分离培训内容", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(tTsFlplanContentService.deleteTTsFlplanContentByIds(ids));
    }
}
