package com.ruoyi.project.training.newstaff.controller;

import java.util.List;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.framework.aspectj.lang.annotation.Log;
import com.ruoyi.framework.aspectj.lang.enums.BusinessType;
import com.ruoyi.project.training.newstaff.domain.TTnSchoolplanTmpl;
import com.ruoyi.project.training.newstaff.service.ITTnSchoolplanTmplService;
import com.ruoyi.framework.web.controller.BaseController;
import com.ruoyi.framework.web.domain.AjaxResult;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.framework.web.page.TableDataInfo;

/**
 * 校招培训模版Controller
 *
 * @author ruoyi
 * @date 2023-07-05
 */
@RestController
@RequestMapping("/newstaff/schoolplantmpl")
public class TTnSchoolplanTmplController extends BaseController
{
    @Autowired
    private ITTnSchoolplanTmplService tTnSchoolplanTmplService;

    /**
     * 查询校招培训模版列表
     */
    @PreAuthorize("@ss.hasPermi('newstaff:schoolplantmpl:list')")
    @GetMapping("/list")
    public TableDataInfo list(TTnSchoolplanTmpl tTnSchoolplanTmpl)
    {
        startPage();
        List<TTnSchoolplanTmpl> list = tTnSchoolplanTmplService.selectTTnSchoolplanTmplList(tTnSchoolplanTmpl);
        return getDataTable(list);
    }

    /**
     * 导出校招培训模版列表
     */
    @PreAuthorize("@ss.hasPermi('newstaff:schoolplantmpl:export')")
    @Log(title = "校招培训模版", businessType = BusinessType.EXPORT)
    @GetMapping("/export")
    public AjaxResult export(TTnSchoolplanTmpl tTnSchoolplanTmpl)
    {
        List<TTnSchoolplanTmpl> list = tTnSchoolplanTmplService.selectTTnSchoolplanTmplList(tTnSchoolplanTmpl);
        ExcelUtil<TTnSchoolplanTmpl> util = new ExcelUtil<TTnSchoolplanTmpl>(TTnSchoolplanTmpl.class);
        return util.exportExcel(list, "schoolplantmpl");
    }

    /**
     * 获取校招培训模版详细信息
     */
    @PreAuthorize("@ss.hasPermi('newstaff:schoolplantmpl:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return AjaxResult.success(tTnSchoolplanTmplService.selectTTnSchoolplanTmplById(id));
    }

    /**
     * 新增校招培训模版
     */
    @PreAuthorize("@ss.hasPermi('newstaff:schoolplantmpl:add')")
    @Log(title = "校招培训模版", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody TTnSchoolplanTmpl tTnSchoolplanTmpl)
    {
        return toAjax(tTnSchoolplanTmplService.insertTTnSchoolplanTmpl(tTnSchoolplanTmpl));
    }

    /**
     * 修改校招培训模版
     */
    @PreAuthorize("@ss.hasPermi('newstaff:schoolplantmpl:edit')")
    @Log(title = "校招培训模版", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody TTnSchoolplanTmpl tTnSchoolplanTmpl)
    {
        return toAjax(tTnSchoolplanTmplService.updateTTnSchoolplanTmpl(tTnSchoolplanTmpl));
    }

    /**
     * 删除校招培训模版
     */
    @PreAuthorize("@ss.hasPermi('newstaff:schoolplantmpl:remove')")
    @Log(title = "校招培训模版", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(tTnSchoolplanTmplService.deleteTTnSchoolplanTmplByIds(ids));
    }
}
