package com.ruoyi.project.training.spec.mapper;

import java.util.List;
import com.ruoyi.framework.aspectj.lang.annotation.DataScope;
import com.ruoyi.project.training.spec.domain.TStFeedback;
import com.ruoyi.project.training.spec.domain.TStPlan;
import com.ruoyi.project.training.spec.domain.TStYearplan;

/**
 * 培训计划Mapper接口
 *
 * @author ruoyi
 * @date 2022-04-13
 */
public interface TStPlanMapper
{
    /**
     * 查询季度培训计划列表
     * @param tStFeedback
     * @return
     */
    List<TStPlan> selectSeasonalTStPlanList(TStFeedback tStFeedback);

    /**
     * 查询月度培训计划列表
     * @param tStFeedback
     * @return
     */
    List<TStPlan> selectMonthlyTStPlanList(TStFeedback tStFeedback);

    /**
     * 根据STAFF_ID查询培训计划列表
     * @autor 王子文
     * @date 2022年4月20日
     * @param tStPlan 培训计划对象
     * @return 培训计划列表
     */
    List<TStPlan> selectTStPlanListByStaffId(TStPlan tStPlan);

    /**
     * 查询培训计划
     *
     * @param id 培训计划ID
     * @return 培训计划
     */
    public TStPlan selectTStPlanById(Long id);

    /**
     * 查询培训计划
     *
     * @param tStPlan 培训计划
     * @return 培训计划
     */
    public TStPlan selectTStPlan(TStPlan tStPlan);

    /**
     * 查询培训计划列表
     *
     * @param tStPlan 培训计划
     * @return 培训计划集合
     */
    public List<TStPlan> selectTStPlanList(TStPlan tStPlan);

    /**
     * 新增培训计划
     *
     * @param tStPlan 培训计划
     * @return 结果
     */
    public int insertTStPlan(TStPlan tStPlan);

    /**
     * 修改培训计划
     *
     * @param tStPlan 培训计划
     * @return 结果
     */
    public int updateTStPlan(TStPlan tStPlan);

    /**
     * 删除培训计划
     *
     * @param id 培训计划ID
     * @return 结果
     */
    public int deleteTStPlanById(Long id);

    /**
     * 批量删除培训计划
     *
     * @param ids 需要删除的数据ID
     * @return 结果
     */
    public int deleteTStPlanByIds(Long[] ids);

    /**
     * 根据staffId修改“未完成”的继任者计划状态为“因离职中止”
     * @param staffId
     * @return
     */
    public int updateStudyStateByStaffId(Long staffId);
}
