package com.ruoyi.project.training.spec.service.impl;

import java.util.Date;
import java.util.List;

import com.ruoyi.common.exception.CustomException;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.framework.security.LoginUser;
import com.ruoyi.project.system.domain.SysUser;
import com.ruoyi.project.system.service.impl.SysUserServiceImpl;
import com.ruoyi.project.training.spec.domain.TStFeedback;
import com.ruoyi.project.training.spec.domain.TStYearplan;
import com.ruoyi.project.training.spec.domain.vo.TStPlanImportVO;
import com.ruoyi.project.training.spec.mapper.TStYearplanMapper;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.project.training.spec.mapper.TStPlanMapper;
import com.ruoyi.project.training.spec.domain.TStPlan;
import com.ruoyi.project.training.spec.service.ITStPlanService;

/**
 * 培训计划Service业务层处理
 *
 * @author ruoyi
 * @date 2022-04-13
 */
@Service
public class TStPlanServiceImpl implements ITStPlanService
{
    private static final Logger log = LoggerFactory.getLogger(SysUserServiceImpl.class);

    @Autowired
    private TStPlanMapper tStPlanMapper;

    @Autowired
    private TStYearplanMapper tStYearplanMapper;

    @Override
    public List<TStPlan> selectSeasonalTStPlanList(TStFeedback tStFeedback) {
        return tStPlanMapper.selectSeasonalTStPlanList(tStFeedback);
    }

    @Override
    public List<TStPlan> selectMonthlyTStPlanList(TStFeedback tStFeedback) {
        return tStPlanMapper.selectMonthlyTStPlanList(tStFeedback);
    }

    /**
     * 查询培训计划
     *
     * @param id 培训计划ID
     * @return 培训计划
     */
    @Override
    public TStPlan selectTStPlanById(Long id)
    {
        return tStPlanMapper.selectTStPlanById(id);
    }

    /**
     * 查询培训计划列表
     *
     * @param tStPlan 培训计划
     * @return 培训计划
     */
    @Override
    public List<TStPlan> selectTStPlanList(TStPlan tStPlan)
    {
        return tStPlanMapper.selectTStPlanList(tStPlan);
    }

    /**
     * 新增培训计划
     *
     * @param tStPlan 培训计划
     * @return 结果
     */
    @Override
    public int insertTStPlan(TStPlan tStPlan)
    {
        return tStPlanMapper.insertTStPlan(tStPlan);
    }

    /**
     * 修改培训计划
     *
     * @param tStPlan 培训计划
     * @return 结果
     */
    @Override
    public int updateTStPlan(TStPlan tStPlan)
    {
        return tStPlanMapper.updateTStPlan(tStPlan);
    }

    /**
     * 批量删除培训计划
     *
     * @param ids 需要删除的培训计划ID
     * @return 结果
     */
    @Override
    public int deleteTStPlanByIds(Long[] ids)
    {
        return tStPlanMapper.deleteTStPlanByIds(ids);
    }

    /**
     * 删除培训计划信息
     *
     * @param id 培训计划ID
     * @return 结果
     */
    @Override
    public int deleteTStPlanById(Long id)
    {
        return tStPlanMapper.deleteTStPlanById(id);
    }

    /**
     * 导入培养计划数据
     *
     * @param planList 培养计划数据列表
     * @param isUpdateSupport 是否更新支持，如果已存在，则进行更新数据
     * @param loginUser 操作用户
     * @param yearPlanId 年度培养计划id
     * @return 结果
     */
    @Override
    public String importData(List<TStPlanImportVO> planList, boolean isUpdateSupport, SysUser loginUser, Long yearPlanId) {
        if (StringUtils.isNull(planList) || planList.size() == 0)
        {
            throw new CustomException("导入培养计划数据不能为空！");
        }
        int successNum = 0;
        int failureNum = 0;
        StringBuilder successMsg = new StringBuilder();
        StringBuilder failureMsg = new StringBuilder();
        TStYearplan yearplan =tStYearplanMapper.selectTStYearplanById(yearPlanId);
        for (TStPlanImportVO planImportVO : planList) {
            try {
                TStPlan plan = new TStPlan();
                plan.setYearPlanId(yearPlanId);
                plan.setPlantName(planImportVO.getPlantName());
                TStPlan result = tStPlanMapper.selectTStPlan(plan);
                if (StringUtils.isNull(result))
                {
                    successNum++;
                    plan.setStaffId(yearplan.getStaffId());
                    plan.setStartDate(planImportVO.getStartDate());
                    plan.setEndDate(planImportVO.getEndDate());
                    plan.setClassContent(planImportVO.getClassContent());
                    plan.setCreaterCode(loginUser.getUserId().toString());
                    plan.setCreatedate(new Date());
                    this.insertTStPlan(plan);
                    successMsg.append("<br/>" + successNum + "、培养计划 " + planImportVO.getPlantName() + " 导入成功");
                } else if (isUpdateSupport) {
                    successNum++;
                    result.setStaffId(yearplan.getStaffId());
                    result.setStartDate(planImportVO.getStartDate());
                    result.setEndDate(planImportVO.getEndDate());
                    result.setClassContent(planImportVO.getClassContent());
                    result.setUpdaterCode(loginUser.getUserId().toString());
                    result.setUpdatedate(new Date());
                    this.updateTStPlan(result);
                    successMsg.append("<br/>" + successNum + "、培养计划 " + planImportVO.getPlantName() + " 更新成功");
                } else {
                    failureNum++;
                    failureMsg.append("<br/>" + failureNum + "、培养计划 " + planImportVO.getPlantName() + " 已存在");
                }
            } catch (Exception e) {
                failureNum++;
                String msg = "<br/>" + failureNum + "、培养计划 " + planImportVO.getPlantName() + " 导入失败：";
                failureMsg.append(msg + e.getMessage());
                log.error(msg, e);
            }
        }
        if (failureNum > 0)
        {
            failureMsg.insert(0, "很抱歉，导入失败！共 " + failureNum + " 条数据格式不正确，错误如下：");
            throw new CustomException(failureMsg.toString());
        }
        else
        {
            successMsg.insert(0, "恭喜您，数据已全部导入成功！共 " + successNum + " 条，数据如下：");
        }
        return successMsg.toString();
    }

    /**
     * 根据staffId修改“未完成”的继任者计划状态为“因离职中止”
     * @param staffId
     * @return
     */
    @Override
    public int updateStudyStateByStaffId(Long staffId) {
        return tStPlanMapper.updateStudyStateByStaffId(staffId);
    }
}
